using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Reflection;
using System.Diagnostics;
using System.IO;
using System.Resources;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization.Formatters;
using System.Data;
using Symcon.Utils.MessageLogger;
using Symcon.Lightridge.LRRUI.Controls;
using Symcon.Lightridge.LRRUI;
using Symcon.LightRidge.BusinessObjects;
using Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.LightRidge.Options;
using Syncfusion.Windows.Forms.Diagram;
namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// Summary description for GraphicsAggregatesUtilities.
	/// </summary>
	public class GraphicsAggregatesUtilities 
	{
        static GraphicsAggregatesUtilities()
		{
			_app = AppDomain.CurrentDomain.GetData("AppObject")	as LRRApp;
			_lrrOptionsObj = _app.GetOptions() as LRROptions;
		}

		#region Public Methods
		
		public static void	UpdateSymbolPropertiesMap(Group smbl,LRRModes lrrModes,Symcon.BusinessObjects.CustomCollection mySelectedProperties, GraphicsManager gm,GraphicsManagers.PropertiesMappingMode propertiesMappingMode,GraphicsManagers.UserMode userMode,Graphics g)
		{
			ArrayList alst = new ArrayList();
			try
            {
				gm.CanvasObj.SuspendLayout();
				Canvas parent = gm.CanvasObj; 
				LRRAggregate myAggregate = GetAggregateForSymbol(smbl); //Selected Aggregate
                if (myAggregate == null && propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
                {
                   
                    //smbl.BackgroundStyle.Color = Color.Black;
                    
                    GraphicsUtilities.RemoveBoundProperties(smbl);
                    return;
                }
				LabelController	myLabelController =	gm.LabelControllerObj; 
				Group mySymbol= smbl;
                //symbol
                //PathNode mySymbol;
				LRRModes myModes = lrrModes; // slected Modes
                if (mySymbol.GetType() == typeof(DashBoardControlProperties))
                {
                    SetBindingForDashBoardControls(mySymbol, lrrModes, mySelectedProperties, gm, propertiesMappingMode, userMode, g);
                    return;
                }
				Syncfusion.Windows.Forms.Diagram.LabelCollection myLabels =	mySymbol.Labels; // Labells for symbol

				for(int	i =	mySymbol.Labels.Count; i>0;i--)
				{
					LRRLabel lrrLabel =	 mySymbol.Labels[i-1] as LRRLabel;
					if(userMode  == GraphicsManagers.UserMode.OPERATOR)
					{
						//lrrLabel.EditStyle.AllowMove = false;
						lrrLabel.ReadOnly = true;
						//lrrLabel.EditStyle.AllowSelect = false;
					}
					if(lrrLabel.Tag	!= null	&& lrrLabel.Tag.GetType() == typeof(IPropertyValue))
					{
						gm.LRRBindingManagerObj.RemoveBinding(lrrLabel.Tag,"PropertyValue",mySymbol.Labels[i-1],true);
					}

				}

				//Set the Header Label
				LRRLabel hsmblLbl = mySymbol.Labels[0] as LRRLabel;
				//hsmblLbl.Visible = true;
				//hsmblLbl.ReadOnly = false;
				if(propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.NORMAL)
				{
					hsmblLbl.Text =	myAggregate.Name;
					hsmblLbl.Name =	myAggregate.Name;
					hsmblLbl.Visible = _app.GetOptions().GraphicsLblsVisibility[0];
					InitLabelLookFeel(hsmblLbl);
				}
				int	count =	0;
				int lblvisibilityIndex = 0;
				//Stack the arraylist with three rows to hold the Labels
				for(int	i =	0 ;	i <	3; i++)
				{
					alst.Add(null);
				}
				//Sort the Modes in Type so they appear in order
				myModes.SortColumn = "ModeTypeID";
				myModes.Sort();

				//Iterate through Selected Modes and Setup the arraylist for each Mode
				foreach(LRRMode	lrmd in	myModes)
				{
			
					int	modeTypeId;
					if(lrmd.ModeTypeID == null || lrmd.ModeTypeID == "")
						continue;
					modeTypeId = Int32.Parse(lrmd.ModeTypeID,Symcon.BusinessObjects.SymConverter.Culture)-1;
					alst[modeTypeId] = lrmd;
				}
				Font myFont	= new Font(smbl.Labels[1].FontStyle.Family,smbl.Labels[1].FontStyle.Size);
				float offsetXPropName =	-1f;
				float offsetXPropValue = -1f;
				float offsetXUOM = -1f;
				if(propertiesMappingMode ==	GraphicsManagers.PropertiesMappingMode.NORMAL)
				{
					GetOffsetLength(mySelectedProperties,myModes,myFont,out	offsetXPropName,out	offsetXPropValue,out offsetXUOM,g);
					if(offsetXPropName <40)
					{
						offsetXPropName = 40;
					}
					else if(offsetXPropName >100)
					{
						offsetXPropName = 100;

					}
					if(offsetXPropValue <40)
					{
						offsetXPropValue = 40;
					}
					else if(offsetXPropValue >100)
					{
						offsetXPropValue =100;

					}
					if(offsetXUOM <20)
					{
						offsetXUOM = 20;
					}
					else if(offsetXUOM >100)
					{
						offsetXUOM =100;

					}
					offsetXPropName	= -offsetXPropName ;
					offsetXPropValue = -offsetXPropValue ;
					offsetXUOM = -offsetXUOM;
				}

				//Iterate through the selected Properties for Aggregate
				//1>Add labels to the symbol
				//2>Add Binding to each Label
				IEnumerator propsEnum =  mySelectedProperties.GetEnumerator();
				while(propsEnum.MoveNext())
				{
					LRRPlantItemProperty plProp = propsEnum.Current as LRRPlantItemProperty;
				
					//if(myProp.IsSelected)
					//{
					if(plProp != null)
					{
						//InitializeLabels(plProp, count,mySymbol, alst,gm,propertiesMappingMode,mySelectedProperties,userMode,offsetXPropName,offsetXPropValue,offsetXUOM,ref lblvisibilityIndex);
                       
						if(plProp.IsExposed || userMode == GraphicsManagers.UserMode.IMPLEMENTOR)
							AddtoPropertyGrid(plProp, alst,	gm,  mySymbol);
						count++;
						lblvisibilityIndex++;
					}
				}

			}
			catch(Exception	ex)
			{
				Logger.LogMessage(typeof(GraphicsSiteComponentUtilities),"UpdateSymbolPropertiesMap :"+ex.Message  ,Logger.Level.Debug);
			}
			finally
			{
				alst.Clear();
				flagRemove = true;
				//gm.CanvasObj.View.Update();
				gm.CanvasObj.ResumeLayout();
                
				_labelindex	= 1;
			}
		}

        private static void SetBindingForDashBoardControls(Group smbl, LRRModes lrrModes, Symcon.BusinessObjects.CustomCollection mySelectedProperties, GraphicsManager gm, GraphicsManagers.PropertiesMappingMode propertiesMappingMode, GraphicsManagers.UserMode userMode, Graphics g)
        {
            //TypeCast Group to Node
            Node nds = smbl as Node;
            Symcon.Lightridge.LRRUI.Controls.DashBoardControlProperties baseObj = nds as Symcon.Lightridge.LRRUI.Controls.DashBoardControlProperties;
            //((PathNode)this.gms.ActiveManager.CanvasObj.Controller.NodesHit.Last).Labels[0] as LRRLabel;
            ControlNode cNode = smbl.Nodes.Last as ControlNode;
            
            //ControlNode cNode = smbl.Nodes["ControlNode"] as ControlNode;
            Control objControl = cNode.HostingControl;
            Symcon.LightRidge.PEAdvisor.DashBoard.BindingInfo bindingInfo = new Symcon.LightRidge.PEAdvisor.DashBoard.BindingInfo();
            string sourceObjectName = "";
            string sourceObjectID = "";
            string sourceProperty = "PropertyValue";
            LRRAggregateProperty aggProp = null;
            LRRAggregateGroupedProperty aggGrpProp = null;
            LRRMode mode = null;
            Symcon.Lightridge.LRRUI.PropertyTable PT = baseObj.PT;
            string sourcePropertyID = "";
            string sourcePropertyModeId = "";
            PropertySpec boundPropSpec = null;
            string[] splitStrings = null;
            char[] ch1 = new char[1];
            ch1[0] = '(';
            foreach (PropertySpec ps in PT.Properties)
            {
                if (ps.Category == "Bound Properties")
                {
                    boundPropSpec = ps;
                    string sPropName = ps.Name;
                    sPropName = sPropName.Remove(sPropName.LastIndexOf(')'));
                    splitStrings = sPropName.Split(ch1);
                    sourcePropertyModeId = splitStrings[1].ToString();
                    bindingInfo.ModeName = sourcePropertyModeId;
                    sourcePropertyModeId = LRRModel.Modes.GetModeByName(sourcePropertyModeId).ModeId;
                    break;
                }
            }
            LRRPlantItemProperty plantItemProperty = mySelectedProperties[0] as LRRPlantItemProperty;
            if (plantItemProperty.GroupPropertyID == null || plantItemProperty.GroupPropertyID.Trim() == "")
            {
                sourcePropertyID = boundPropSpec.ObjectRef.ToString();
                aggProp = LRRModel.AggregatePropertiesObj.GetAggregatePropFromID(sourcePropertyID);
                bindingInfo.SelectedSiteComponentProperties.Add(aggProp);
                bindingInfo.SelectedModes.Add(LRRModel.ProjectModes.GetModeByID(sourcePropertyModeId));
                bindingInfo.SourceProperty = "PropertyValue";
                sourceObjectName = "LRRAggregatePropertyValue";
                bindingInfo.SourceObjectName = "LRRAggregatePropertyValue";
                LRRAggregatePropertyValue aggPropValue = LRRModel.AggregatePropertyValuesObj.GetAggregateValueObjectForPropertyMode(sourcePropertyModeId,sourcePropertyID);
                bindingInfo.SourceObjectID = aggPropValue.ID;
                sourceObjectID = aggPropValue.ID;
                bindingInfo.SiteComponentProperty = aggProp.Name;
                bindingInfo.SiteComponentName = baseObj.PlantItem.Name;
            }
            else
            {
                string currentLookupQualifierID = "";
                string currentGroupedPropertyID = "";
                if (boundPropSpec.ObjectRef.ToString().IndexOf('|') != -1)
                {
                    string[] ids = boundPropSpec.ObjectRef.ToString().Split("|".ToCharArray());
                    currentLookupQualifierID = ids[0];
                    currentGroupedPropertyID = ids[1];
                }
                string attName = boundPropSpec.Name;
                sourceObjectName = "LRRAggregateGroupedPropertyValueID";
                bindingInfo.SourceObjectName = "LRRAggregateGroupedPropertyValueID";
                bindingInfo.SourceProperty = "PropertyValue";
                LRRAggregateGroupedPropertyValue aggGrpPropValue = LRRModel.AggregateGroupedPropertyValuesObj.GetGroupedValueObjectForModeSiteCompAndLookUpProp(currentGroupedPropertyID, currentLookupQualifierID, sourcePropertyModeId);
                bindingInfo.SourceObjectID = aggGrpPropValue.AggregateGroupedPropertyValueID;
                sourceObjectID = aggGrpPropValue.AggregateGroupedPropertyValueID;
                aggGrpProp = LRRModel.AggregateGroupedPropertiesObj.GetAggregateGroupPropFromID(currentGroupedPropertyID);
                bindingInfo.SelectedSiteCompGroups.Add(aggGrpProp);
                bindingInfo.SiteComponentProperty = boundPropSpec.Name;
                bindingInfo.SiteComponentName = baseObj.PlantItem.Name;
                bindingInfo.SelectedModes.Add(LRRModel.ProjectModes.GetModeByID(sourcePropertyModeId));
            }
            objControl.Tag = bindingInfo;
            object sourceObject = gm.GetSourceObject(sourceObjectName, sourceObjectID);
            LRRLabel lbl = smbl.Labels[0] as LRRLabel;
            lbl.Tag = objControl;
            gm.LRRBindingManagerObj.AddBinding(sourceObject, "PropertyValue", boundPropSpec, "DefaultValue", true);
            gm.LRRBindingManagerObj.AddBinding(sourceObject, "PropertyValue", lbl, "Text", true);
            mode = LRRModel.ChildModel.Modes.GetModeByID(sourcePropertyModeId);
            InitializeLablesForDashBoard(objControl, smbl, aggProp, aggGrpProp, mode);
        }

        public static void InitializeLablesForDashBoard(Control objControl, Group smbl, LRRAggregateProperty aggregateProp, LRRAggregateGroupedProperty aggregateGrpProp, LRRMode mode)
        {
            string aggregate = "";
            string aggregatePropName = "";
            string modeName = "";
            string myUOMName = "";
            string modelUOMID = "";
            string uiUOMIDID = "";
            string parentPropID = "";
            LRRUOMType uomTypeObj = null;
            LRRLabel lblAggregate = smbl.Labels[4] as LRRLabel;
            LRRLabel lblMode = smbl.Labels[6] as LRRLabel;
            LRRLabel lblUom = smbl.Labels[9] as LRRLabel;
            LRRLabel lblParent = smbl.Labels[1] as LRRLabel;
            LRRLabel lblPropValue = smbl.Labels[2] as LRRLabel;

            object agg = objControl.GetType().GetProperty("SourceItem").GetValue(objControl, null);
            if (aggregateProp != null)
            {
                parentPropID = aggregateProp.ID;
                uomTypeObj = LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(aggregateProp.UOMTypeID);
                LRRAggregatePropertyValue  aggregatePropValue = LRRModel.AggregatePropertyValuesObj.GetAggregateValueObjectForPropertyMode(mode.ModeId,parentPropID);
                lblPropValue.Tag = aggregatePropValue;
                lblPropValue.ID = aggregatePropValue.ID;
                lblPropValue.ParentID = parentPropID;
                lblPropValue.Name = aggregateProp.Name + "(" + mode.ModeName + ")";
            }
            else if (aggregateGrpProp != null)
            {
                parentPropID = aggregateGrpProp.AggregateGroupedPropertyID;
                LRRPropertyType propType = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(aggregateGrpProp.PropertyID);
                string UomTypeID = propType.UOMTypeID;
                uomTypeObj = LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(UomTypeID);
                LRRAggregateGroupedPropertyValue gpValue = LRRModel.AggregateGroupedPropertyValuesObj.GetAggGroupValueFromAggGroupPropIDandModeID(aggregateGrpProp.AggregateGroupedPropertyID, mode.ModeId);
                lblPropValue.Tag = gpValue;
                lblPropValue.ID = gpValue.AggregateGroupedPropertyValueID;
                lblPropValue.ParentID = parentPropID;
                
            }
            if (uomTypeObj != null)
            {
                string modelID = LRRModel.MeasurementSystems.GetCurrentModelUOM().ID;
                string localID = _app.AppLocaleID;
                modelUOMID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(modelID, uomTypeObj.UOMTypeID);
                uiUOMIDID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(localID, uomTypeObj.UOMTypeID);
            }
            if (agg != null)
            {
                aggregate = agg.ToString();
            }
            if (mode != null)
            {
                modeName = mode.ModeDisplayName;
            }
            object aggProp = objControl.GetType().GetProperty("Attribute").GetValue(objControl, null);
            if (aggProp != null)
            {
                aggregatePropName = aggProp.ToString();
            }

            //Parent Label.
            lblParent.ParentID = parentPropID;
            lblParent.ID = parentPropID;
            lblParent.IsParent = true;
            if (aggregateGrpProp != null)
            {
                lblParent.Tag = aggregateGrpProp;
            }
            else
            {
                lblParent.Tag = aggregateProp;
            }

            Graphics g = objControl.CreateGraphics();
            //SiteComponent  Label
            lblAggregate.Text = aggregate;
            lblAggregate.Visible = true;
            SizeF sizeSiteComponent = g.MeasureString(aggregate, objControl.Font);
            if (sizeSiteComponent.Width > 150)
            {
                lblAggregate.Size = new SizeF(150, lblAggregate.OffsetY);
            }
            else
            {
                lblAggregate.Size = new SizeF(sizeSiteComponent.Width + 15, lblAggregate.OffsetY);
            }

            //Mode Label
            SizeF sizeMode = g.MeasureString(modeName, objControl.Font);
            if (sizeMode.Width > 150)
            {
                lblMode.Size = new SizeF(150, lblMode.OffsetY);
                
            }
            else
            {
                lblMode.Size = new SizeF(sizeMode.Width + 5, lblMode.OffsetY);
            }
            lblMode.Text = modeName;
            lblMode.Visible = true;

            //UOM Label
            lblUom.Tag = uomTypeObj;
            lblUom.Visible = true;

            myUOMName = GraphicsUtilities.GetUomName(uiUOMIDID);
            SizeF sizeUOM = g.MeasureString(myUOMName, objControl.Font);
            if (sizeUOM.Width > 150)
            {
                lblUom.Size = new SizeF(150, lblUom.OffsetY);
            }
            else
            {
                lblUom.Size = new SizeF(sizeUOM.Width + 5, lblUom.OffsetY);
            }
            lblUom.Text = myUOMName;
            lblUom.Name = "UOM" + "(" + myUOMName + ")-" + aggregatePropName;
            lblUom.ID = uiUOMIDID;
            lblUom.IsUOM = true;
            lblUom.ParentID = parentPropID;
            lblUom.LabelDoubleClick -= new NodeMouseEventHandler(lblUom_LabelDoubleClick);
            lblUom.LabelDoubleClick += new NodeMouseEventHandler(lblUom_LabelDoubleClick);
             
            
        }

        static void lblUom_LabelDoubleClick(NodeMouseEventArgs evtArgs)
        {
            //LRRLabel lbl = evtArgs.Node as LRRLabel;
            LRRLabel lbl = ((PathNode)evtArgs.Node).Labels[0] as LRRLabel;
            if (lbl.ID != null && lbl.IsUOM && lbl.Tag != null)
            {
                LRRUOMType lrrUOMType = lbl.Tag as LRRUOMType;
                if (lrrUOMType != null)
                    GraphicsUtilities.ChangeUOMClick(lbl, lrrUOMType.UOMTypeID);
            }
        }



		private	static void	GetOffsetLength(Symcon.BusinessObjects.CustomCollection mySelectedProperties,LRRModes mySelectedModes,Font myFont,out float offsetXPropName,out float offsetXPropValue,out float	offsetXUOM,Graphics	gphs)
		{

			offsetXPropName	= -1;
			offsetXPropValue = -1;
			offsetXUOM = -1;
			SizeF stringSize;
			IEnumerator propsEnum =  mySelectedProperties.GetEnumerator();
			while(propsEnum.MoveNext())
			{
				LRRPlantItemProperty plProp = propsEnum.Current as LRRPlantItemProperty;
				IProperty myProp = null;
				if(plProp.ID != null)
				{
					myProp = LRRModel.AggregatePropertiesObj.GetAggregatePropFromID(plProp.ID);
					if(myProp != null)
					{
						stringSize = gphs.MeasureString(myProp.GetName(),myFont);
						if(stringSize.Width>offsetXPropName)
						{
							offsetXPropName	= stringSize.Width;
						}
						foreach(LRRMode	lrrmode	in mySelectedModes)
						{
							LRRAggregatePropertyValue lrrAgpv = myProp.GetPropertyValueForMode(lrrmode.ModeId) as LRRAggregatePropertyValue;
							if(lrrAgpv == null)
								continue;
							string myPropValue = lrrAgpv.PropertyValue;; // myProp.GetPropertyValueForMode(lrrmode.ModeId).PropertyValue;
							stringSize = gphs.MeasureString(myPropValue,myFont);
							if(stringSize.Width>offsetXPropValue)
							{
								offsetXPropValue = stringSize.Width;
							}

						}
						string UOMId = myProp.GetUOM();
						if(UOMId !=	null &&	UOMId.Trim() !=	"")
						{
							string UOMName = LRRModel.ChildModel.GetUOMNameFromID(UOMId);
							stringSize = gphs.MeasureString(UOMName,myFont);
							if(stringSize.Width>offsetXUOM)
							{
								offsetXUOM = stringSize.Width;
							}
						}
					}
				}
			}
		}

//		public static void UnwirePropertyValueEvent(GraphicsManager gm)
//		{
////			Canvas activeCanvas = gm.CanvasObj;
////			//LRRModes selectedModes = gms.SelectedModes;
////
////
////			Symbol mySymbol	= null;
////			
////		
////			//Iterate Through Each Symbol and add to snc
////			foreach(INode nde in activeCanvas.Model.Nodes)
////			{
////				mySymbol = nde as Symbol;
////				string propName = null;
////				if(mySymbol	!= null	&& mySymbol.GetPropertyContainerByName("SiteItem") != null && mySymbol.GetPropertyContainerByName("SiteItem").ToString() != "Select")
////				{
////
////					if(mySymbol.GetPropertyContainerByName("SiteItem").ToString().EndsWith("]"))
////					{
////						propName = mySymbol.GetPropertyContainerByName("SiteItem").ToString().Substring(0,mySymbol.GetPropertyContainerByName("SiteItem").ToString().IndexOfAny("[".ToCharArray()));
////					}
////					else
////					{
////						propName = mySymbol.GetPropertyContainerByName("SiteItem").ToString();
////					}
////
////					if(mySymbol.GetPropertyContainerByName("ItemType") != null	&& (string)mySymbol.GetPropertyContainerByName("ItemType") !=	"Select")
////					{
////						PropertyTable pt =  GetPropertyTableForsymbol(mySymbol);
////						Symcon.LightRidge.BusinessObjects.LRRAggregates	lrrAggregates = LRRModel.AggregatesObj;
////						LRRAggregate lrrAggregate = lrrAggregates.GetAggregateFromName(propName);
////						LRRAggregateProperties lrrAggregateProps =	lrrAggregate.AggregateProperties;
////						foreach(LRRAggregateProperty lrrAggregateProp in lrrAggregateProps)
////						{
////							foreach(LRRMode mode in LRRModel.ProjectModes)
////							{
////								LRRAggregatePropertyValue lrrAggregatePropertyValue = lrrAggregateProp.GetValueObjectForPropertyMode(mode.ModeId);
////								//lrrscpv.PropertyValueChanged-=new PropertyChanged(gm.PropertyValueChanged);
////							}
////
////
////						}
////						//	UpdateSymbolPropertiesMap(openedViewSymbol,selectedModes,lrrscps,gm);
////					}
////				}
////			}
//
//
//		}


		#endregion 

		#region Private Methods

		private static void	AddtoPropertyGrid(LRRPlantItemProperty lrrAggregateProperty, ArrayList myModes,GraphicsManager gm, Group mySymbol)
		{
            if (lrrAggregateProperty.ID == null && lrrAggregateProperty.Name != "NullValue")
            {
                GraphicsUtilities.RemoveBoundProperties(mySymbol);
                return;
            }

			if(lrrAggregateProperty.Name == "NullValue")
			{
				return;
			}
			PropertyTable myPT = GetPropertyTableForsymbol(mySymbol);
			ArrayList arryList = new ArrayList();
			//ArrayList arLstBoundPropsIndex = new ArrayList();
			LabelController myLabelController = gm.LabelControllerObj;
			LRRBindingManager lrrBindingManager = gm.LRRBindingManagerObj;
			LRRAggregateProperty myAggregateProp = LRRModel.AggregatePropertiesObj.GetAggregatePropFromID(lrrAggregateProperty.ID);

			//Remove al Bound Properties from the Property Table
			if(flagRemove)
			{
				for(int i= myPT.Properties.Count-1; i>0;i--)
				{
					PropertySpec ps = myPT.Properties[i] as PropertySpec;
					if(ps.Category == "Bound Properties")
					{
						myPT.Properties.Remove(ps);
					}
				}
				flagRemove = false;
			}

			try
			{
				LRRLabel uomLabel  = null;
//				if(propertiesMappingMode ==	GraphicsManagers.PropertiesMappingMode.OPEN)
//				{
					uomLabel = GraphicsUtilities.GetUOMLabelFromParentID(mySymbol,lrrAggregateProperty.SerializedID); 
//				}
				foreach(LRRMode	myMode in myModes)
				{
					if(myMode != null)
					{
						foreach(DictionaryEntry	obj	in myLabelController.SymbolObjectBinding)
						{
							//Associate	the	key	object to Labellookup Type
							LabelLookup	lblLookup =	obj.Key	as LabelLookup;

							if(myLabelController.SymbolObjectBinding.ContainsKey(lblLookup.PropertyName))
							{
								PropertySpec ps = myLabelController.SymbolsCollection[lblLookup]	as PropertySpec;
								//	myPT.Properties.Remove(objct);
								myLabelController.SymbolsCollection.Remove(lblLookup);
								arryList.Add(lblLookup);
								
							}
						}
						for(int	i=0; i<arryList.Count; i++)
						{
							myLabelController.SymbolObjectBinding.Remove(arryList[i]);
						}
						arryList.Clear();
					}
				}
				
				
				foreach(LRRMode	myMode in myModes)
				{
					if(myMode != null)
					{
						LabelLookup	lblLookup  = new LabelLookup();
						lblLookup.SymbolID = mySymbol.GetPropertyContainerByName("ID").ToString();
						lblLookup.PropertyName = lrrAggregateProperty.Name + "(" + myMode.ModeName + ")" ;
						LRRAggregatePropertyValue lrrAgrPropValObj =  LRRModel.AggregatePropertyValuesObj.GetAggregateValueObjectForPropertyMode(myMode.ModeId,lrrAggregateProperty.ID);
						if(lrrAgrPropValObj != null)
						{
							if(uomLabel != null)
							{
								string uomid = lrrAgrPropValObj.GetUOMID();
                                lblLookup.PropertyValue = lrrAgrPropValObj.GetPropertyValueForUOM(uomid,uomLabel.ID,lrrAgrPropValObj.GetPropertyValue());  
							}
							else
							{
								string uomid = lrrAgrPropValObj.GetUOMID();
                               	lblLookup.PropertyValue = lrrAgrPropValObj.GetPropertyValueForUOM(uomid,myAggregateProp.GetUIUOMID(),lrrAgrPropValObj.GetPropertyValue());  
							}
						}
						lblLookup.PropertyMode = myMode.ModeId;
						lblLookup.PropertyID = lrrAggregateProperty.ID;

						//LRRAggregatePropertyValue lrrAggregatePropertyValue	= LRRModel.AggregatePropertyValuesObj.GetAggregateValueObjectForPropertyMode(myMode.ModeId,lrrAggregateProperty.ID);
						if(lrrAgrPropValObj !=	null &&	!myLabelController.SymbolsCollection.ContainsKey(lblLookup))
						{
							int	propIndex;
							if(uomLabel != null)
							{
								string uomid = lrrAgrPropValObj.GetUOMID();
								string propValue = lrrAgrPropValObj.GetPropertyValueForUOM(uomid,uomLabel.ID,lrrAgrPropValObj.GetPropertyValue());
								propIndex =	myPT.Properties.Add(new	PropertySpec(lrrAggregateProperty.Name	+ "(" +	myMode.ModeName	+ ")" ,typeof(String),"Bound Properties",myAggregateProp.GetName(),propValue,"",typeof(String), lrrAggregateProperty.SerializedID));
							}
							else
							{
								string uomid = lrrAgrPropValObj.GetUOMID();
								string propValue = lrrAgrPropValObj.GetPropertyValueForUOM(uomid,myAggregateProp.GetUIUOMID(),lrrAgrPropValObj.GetPropertyValue());
								propIndex =	myPT.Properties.Add(new	PropertySpec(lrrAggregateProperty.Name	+ "(" +	myMode.ModeName	+ ")" ,typeof(String),"Bound Properties",myAggregateProp.GetName(),propValue,"",typeof(String), lrrAggregateProperty.SerializedID));
							}
							Attribute[]	attrib = new Attribute[1];
							ReadOnlyAttribute readonlyattrib;
							readonlyattrib = new ReadOnlyAttribute(!lrrAgrPropValObj.GetEditableState());
							attrib[0] =	readonlyattrib;
							myPT.Properties[propIndex].AttributesNew = attrib;

							//int	propIndex =	myPT.Properties.Add(new	PropertySpec(lrrAggregateProperty.Name	+ "(" +	myMode.ModeName	+ ")" ,typeof(String),"Bound Properties",lrrAggregateProperty.Description,lrrAgrPropValObj.PropertyValue,"",typeof(String), lrrAggregateProperty.ID));
							//lrrBindingManager.AddBinding(lrrAgrPropValObj, "PropertyValue",  myPT.Properties[propIndex], "DefaultValue");
							PropertySpec ps	= myPT.Properties[propIndex] as	PropertySpec;
							//lrrBindingManager.AddBinding(ps, "DefaultValue",lrcpv, "PropertyValue");
							myLabelController.SymbolsCollection.Add(lblLookup,ps );
							lrrAgrPropValObj.PropertyValueChanged+=new PropertyChanged(gm.PropertyValueChanged);
						}
						myLabelController.SymbolObjectBinding.Add(lblLookup,mySymbol); 
						//gm.CanvasObj.View.Refresh();
                        gm.CanvasObj.View.RefreshPageSettings();
                        
					}
				}
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(GraphicsManagers),"AddtoPropertyGrid :"+ex.Message  ,Logger.Level.Debug);
			}
		}

		//
		//To Do: Add offset and visibility Logic
		//

		/// <summary>
		/// Create a matrix for the selected Property and add label 
		/// at defined positions
		///	00(Nothing)					01(PropValue 1)		02(Nothing)
		///	10(PropName)				11(PropValue 2)		12(UOM)
		///	20(Property	Description)	21(PropValue 3)		22(Nothing)
		/// </summary>
		/// <param name="myProp">Selected Property</param>
		/// <param name="count">Selected Property Index</param>
		/// <param name="mySymbol">Symbol</param>
		/// <param name="myModes">Selected Modes</param>
		private static void	InitializeLabels(LRRPlantItemProperty plProp, int count,PathNode mySymbol, ArrayList myModes,GraphicsManager gm,GraphicsManagers.PropertiesMappingMode propertiesMappingMode,Symcon.BusinessObjects.CustomCollection selectedProps,GraphicsManagers.UserMode userMode,float offsetXPropName,float offsetXPropValue,float offsetXUOM,ref int lblVsbltyIdx)
		{
			IProperty myProp = null;
			LRRMode							compare1Mode = null;
			IPropertyValue	compare1AggrValueObj = null;
			LRRMode							refMode	= null;
			IPropertyValue	refAggrValueObj	= null;
			LRRMode							compare2Mode = null;
			IPropertyValue	compare2AggrValueObj = null; 
			LRRBindingManager lrrBindingManager	= gm.LRRBindingManagerObj;
			int	_maxRow	= 3;
			int	_maxCol	= 3;
			double currentTolerence = 0;
            double currentRefValue = 0;
			bool isCompare1Visible = true;
            bool isCompare2Visible = true;
			double currentCompare1Value = 0;
            double currentCompare2Value = 0;
            double minValue = 0;
            double maxValue = 0;
            string modelUOMID = null;
            string uiUOMIDID = null;
			LRRUOMType uomTypeObj =	null;
			//	float wdth = 0;
			if(plProp.ID != null)
			{
				myProp = LRRModel.AggregatePropertiesObj.GetAggregatePropFromID(plProp.ID);

				//Set the UOMType Object
				if(myProp.GetUOMType()	!= null)
				{
					uomTypeObj = LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(myProp.GetUOMType());
				}
                if (uomTypeObj != null)
                {
                    string modelID = LRRModel.MeasurementSystems.GetCurrentModelUOM().ID;
                    string localID = _app.AppLocaleID;
                    modelUOMID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(modelID, uomTypeObj.UOMTypeID);
                    uiUOMIDID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(localID, uomTypeObj.UOMTypeID);
                }
				foreach(LRRMode mode in myModes)
				{
					if(mode != null)
					{
						//Set Compare1ValueObject
						if(mode.ModeTypeID == "1")
						{
							compare1Mode =	mode;		
							IPropertyValue iPropVal = SetValueObjects(mode, ref currentCompare1Value, uomTypeObj, myProp, plProp);
							//compare1AggrValueObj = myProp.GetPropertyValueForMode(mode.ModeId) as IPropertyValue;
							compare1AggrValueObj = iPropVal;
						}
						else if(mode.ModeTypeID == "2")
						{
							refMode =	mode;
							   IPropertyValue iPropVal = SetValueObjects(mode, ref currentRefValue, uomTypeObj, myProp, plProp);
							//refAggrValueObj = myProp.GetPropertyValueForMode(mode.ModeId) as IPropertyValue;
							refAggrValueObj =iPropVal;
						}
						else if(mode.ModeTypeID == "3")
						{
							compare2Mode =	mode;		
							IPropertyValue iPropVal = SetValueObjects(mode, ref currentCompare2Value, uomTypeObj, myProp, plProp);
						//	compare2AggrValueObj = myProp.GetPropertyValueForMode(mode.ModeId) as IPropertyValue;
							compare2AggrValueObj =iPropVal;
						}
					}
				}

				
                //Set Min tolerance	and	Max	tolerance Value
                if (uomTypeObj != null && uomTypeObj.Tolerance != null && !uomTypeObj.IsEVV)
                {
                    currentTolerence = (double)Symcon.BusinessObjects.SymExcelConverter.Convert(typeof(string), uomTypeObj.Tolerance, typeof(double));//Decimal.Parse(uomTypeObj.Tolerance);
                    minValue = currentRefValue - (currentRefValue * currentTolerence);
                    maxValue = currentRefValue + (currentRefValue * currentTolerence);

                }

			   //Set Visibility variables based on	tolerance
             if (!_lrrOptionsObj.ShowLabels)
             {
                 if (uomTypeObj != null && !uomTypeObj.IsEVV)
                 {
                     if ((currentCompare1Value >= maxValue || currentCompare1Value <= minValue) && currentRefValue != 0) 
                        {
                            isCompare1Visible = true;
                        }
                        else
                        {
                            isCompare1Visible = false;
                        }

                        if ((currentCompare2Value >= maxValue || currentCompare2Value <= minValue) && currentRefValue != 0)
                        {
                            isCompare2Visible = true;
                        }
                        else
                        {
                            isCompare2Visible = false;
                        }
                 }
             }

			}

			if(plProp.Name == "NullValue" || (!plProp.IsExposed && userMode != GraphicsManagers.UserMode.IMPLEMENTOR))
			{
				for(int i = 0 ;i< ((mySymbol.Labels.Count-2)/4);i++)
				{
					
					LRRLabel lbl =  mySymbol.Labels[_labelindex] as LRRLabel;					
					lbl.Text = "";
					lbl.ParentID = "";
					lbl.BackgroundStyle.Color = Color.Transparent;
					lbl.BackgroundStyle.Color = Color.Black;
                    
                    
					_labelindex++;
				}
				lblVsbltyIdx--;
				return;
			}

			LRRLabel uomLabel = null;

            if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN || propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.NONE)
			{
                uomLabel = GraphicsUtilities.GetUOMLabel(mySymbol.Labels[_labelindex] as LRRLabel);
                LRRUOMs lrrUOMs = LRRModel.ChildModel.GetUOMsFromUOMTypeID(uomTypeObj.UOMTypeID);
                bool flagUOMTypeMisMatch = true;
                foreach (LRRUOM lrrUOM in lrrUOMs)
                {
                    if (uomLabel.ID == lrrUOM.ID)
                    {
                        flagUOMTypeMisMatch = false;
                        break;
                    }
                }

                if (uomLabel != null && flagUOMTypeMisMatch)
                {
                    //Change in Persisted UOM in case of change in UOMTYPE in Master Attributes
                    string myUOMName = GraphicsUtilities.GetUomName(uiUOMIDID);
                    //string myUOMName = GetUomName(refPropValueObj.GetUOMID());
                    uomLabel.Text = myUOMName;
                    uomLabel.Name = "UOM" + "(" + myUOMName + ")-" + myProp.GetName();
                    uomLabel.ParentID = plProp.SerializedID;
                    uomLabel.ID = uiUOMIDID;
                    uomLabel.IsUOM = true;
                }
			}

			for	(int i = 0;	i <	_maxRow; i++)
			{
				for	(int j = 0;	j <	_maxCol; j++)
				{		
					string str = (string)i.ToString(Symcon.BusinessObjects.SymConverter.Culture) +	j.ToString(Symcon.BusinessObjects.SymConverter.Culture);
					//-------------------------------------------------
					//|	00(Nothing)		01(PropValue 1)		02(Nothing)
					//|	10(PropName)	11(PropValue 2)		12(UOM)
					//|	20(Property	Description)		21(PropValue 3)		22(Nothing)
					//-------------------------------------------------
				
					switch(str)
					{
						case "00":
						{
							break;
						}
						case "01":
						{
							LRRLabel smblLbl = mySymbol.Labels[_labelindex]as LRRLabel;
							smblLbl.Visible = true;
                            float symbolHeight = smblLbl.OffsetY;
							_labelindex++;
							if(compare1AggrValueObj != null)
							{
								if(smblLbl.Tag != null)
								{
									gm.LRRBindingManagerObj.RemoveBinding(smblLbl.Tag,"PropertyValue",smblLbl,true);
                                    gm.LRRBindingManagerObj.RemoveBinding(smblLbl.Tag, "ErrCode", smblLbl, true);
								}
								lrrBindingManager.AddBinding(compare1AggrValueObj,"PropertyValue",smblLbl,"Text");
                                lrrBindingManager.AddBinding(compare1AggrValueObj, "ErrCode", smblLbl, "LabelColorCode");
								if(uomLabel != null)
								{
									string uomid = compare1AggrValueObj.GetUOMID();
                                    string newPropValue = compare1AggrValueObj.GetPropertyValueForUOM(uomid, uomLabel.ID, compare1AggrValueObj.GetPropertyValue());
                                    if(compare1AggrValueObj.GetDataType() == null)
                                        newPropValue = Symcon.LightRidge.PEAdvisor.GridUtilities.Utilities.FormatNumber(newPropValue, _app);
                                    smblLbl.Text = newPropValue;//compare1AggrValueObj.GetPropertyValueForUOM(uomid, uomLabel.ID, compare1AggrValueObj.GetPropertyValue());
								}
								else
								{
									//string uomid = compare1AggrValueObj.GetUOMID();
                                    string newPropValue = compare1AggrValueObj.GetPropertyValueForUOM(modelUOMID,uiUOMIDID , compare1AggrValueObj.GetPropertyValue());
                                    if (compare1AggrValueObj.GetDataType() == null)
                                         newPropValue = Symcon.LightRidge.PEAdvisor.GridUtilities.Utilities.FormatNumber(newPropValue, _app);
                                    smblLbl.Text = newPropValue;// compare1AggrValueObj.GetPropertyValueForUOM(uomid, myProp.GetUIUOMID(), compare1AggrValueObj.GetPropertyValue());
								}
								smblLbl.ID = compare1AggrValueObj.GetID();
								smblLbl.Tag = compare1AggrValueObj;
								smblLbl.IsCompare1 = true;
								if(propertiesMappingMode ==	GraphicsManagers.PropertiesMappingMode.NORMAL)
								{
									smblLbl.OffsetX	= offsetXPropValue + offsetXUOM	- offsetXPropValue/2 /*-offsetXUOM/2*/;
									smblLbl.OffsetX =	-offsetXPropValue;
								}
                                smblLbl.BackgroundStyle.Color = compare1AggrValueObj.GetBackColor();
                                smblLbl.BackgroundStyle.Color = compare1AggrValueObj.GetForeColor();
                                GraphicsUtilities.SetColor(smblLbl, compare1AggrValueObj);
								smblLbl.Name = plProp.Name + "(" + compare1Mode.ModeName +	")";
								 smblLbl.Visible = isCompare1Visible;
							}
							else
							{
								smblLbl.Text = "";
								smblLbl.ID = "";
								smblLbl.Tag	= null;
							}

							
							smblLbl.HorizontalAlignment	= StringAlignment.Far;
							smblLbl.ParentID = plProp.SerializedID;
							//smblLbl.LabelDoubleClick+=new NodeMouseEventHandler(smblLbl_LabelDoubleClick);
							InitLabelLookFeel(smblLbl);
                            if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
                            {
                                smblLbl.OffsetY = symbolHeight;
                                
                            }
							break;
						}
						case "02":
						{
							LRRLabel smblLbl;
							if(mySymbol.Labels.Count < 22 + (selectedProps.Count*2))
							{
								smblLbl = new LRRLabel();
								//smblLbl.Anchor = BoxPosition.TopLeft;
								mySymbol.Labels.Insert(_labelindex,smblLbl);
								//smblLbl.EditStyle.Enabled = true;
								//smblLbl.EditStyle.AllowResize = false; 
								if(propertiesMappingMode ==	GraphicsManagers.PropertiesMappingMode.NORMAL)
								{
									smblLbl.OffsetX	= offsetXUOM -offsetXUOM/2;
								}
								smblLbl.HorizontalAlignment	= StringAlignment.Center;
								smblLbl.VerticalAlignment	= StringAlignment.Far;
								//smblLbl.OffsetX = 10;
                                smblLbl.OffsetX = 10;
                                smblLbl.OffsetY = 10;
								//smblLbl.OffsetY = 10;
								//smblLbl.Translate(mySymbol.Labels[_labelindex - 1].X + mySymbol.Labels[_labelindex - 1].Width + smblLbl.OffsetX - smblLbl.OffsetX,mySymbol.Labels[_labelindex - 1].Y  - smblLbl.OffsetY);
								//smblLbl.LabelDoubleClick+=new NodeMouseEventHandler(smblLbl_LabelDoubleClick);
							}
							else
							{
								smblLbl = mySymbol.Labels[_labelindex]as LRRLabel;
							}
                            float symbolHeight = smblLbl.OffsetY;
							smblLbl.ParentID = plProp.SerializedID;
							_labelindex++;
                            if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
                            {
                                //smblLbl.OffsetY = symbolHeight;
                                smblLbl.OffsetY = symbolHeight;
                            }
							//smblLbl.BackgroundStyle.Color = Color.Red;
							break;
						}
						case "10":
						{
							LRRLabel smblLbl = mySymbol.Labels[_labelindex]as LRRLabel;
						//	smblLbl.Visible = true;
                            float symbolHeight = smblLbl.OffsetY;
							_labelindex++;

							if(propertiesMappingMode ==	GraphicsManagers.PropertiesMappingMode.NORMAL && smblLbl.ParentID != myProp.GetID())
							{
								smblLbl.Text = plProp.Name;
								InitLabelLookFeel(smblLbl);

								smblLbl.OffsetX	= (offsetXPropName+offsetXPropValue+offsetXUOM)-offsetXPropName/2;
								//smblLbl.OffsetX =	-offsetXPropName;
                                smblLbl.OffsetX = -offsetXPropName;
								smblLbl.Visible = _app.GetOptions().GraphicsLblsVisibility[(lblVsbltyIdx * 2) + 1];
							}
							else if(propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.REMAP)
							{
								smblLbl.Text = plProp.Name;
								smblLbl.Visible = _app.GetOptions().GraphicsLblsVisibility[(lblVsbltyIdx * 2) + 1];
							}
							smblLbl.ParentID = plProp.ID;
							smblLbl.ID = myProp.GetID();
							smblLbl.IsParent = true;
							smblLbl.Tag	= myProp;
							if(userMode == GraphicsManagers.UserMode.IMPLEMENTOR)
								smblLbl.ReadOnly = false;
							//smblLbl.LabelDoubleClick+=new NodeMouseEventHandler(smblLbl_LabelDoubleClick);;
                            if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
                            {
                                smblLbl.OffsetY = symbolHeight;
                            }
							break;
						}
						case "11":
						{
							LRRLabel smblLbl = mySymbol.Labels[_labelindex]as LRRLabel;
                            float symbolHeight = smblLbl.OffsetY;
							smblLbl.Visible = true;
							_labelindex++;

							
							if(refAggrValueObj != null)
							{
								if(smblLbl.Tag != null)
								{
									gm.LRRBindingManagerObj.RemoveBinding(smblLbl.Tag,"PropertyValue",smblLbl,true);
                                    gm.LRRBindingManagerObj.RemoveBinding(smblLbl.Tag, "ErrCode", smblLbl, true);
								}
								//LRRMode	mylrm =	myModes[1] as LRRMode;
								//LRRAggregatePropertyValue selPropValue = myProp.GetValueObjectForPropertyMode(mylrm.ModeId);
								//if(selPropValue	!= null)
								//{
								lrrBindingManager.AddBinding(refAggrValueObj,"PropertyValue",smblLbl,"Text");
                                lrrBindingManager.AddBinding(compare1AggrValueObj, "ErrCode", smblLbl, "LabelColorCode");
								if(uomLabel != null)
								{
                                    
									string uomid = refAggrValueObj.GetUOMID();
                                    string newPropValue = refAggrValueObj.GetPropertyValueForUOM(uomid, uomLabel.ID, refAggrValueObj.GetPropertyValue());
                                    if (refAggrValueObj.GetDataType() == null)
                                            newPropValue = Symcon.LightRidge.PEAdvisor.GridUtilities.Utilities.FormatNumber(newPropValue, _app);
                                    smblLbl.Text = newPropValue;// refAggrValueObj.GetPropertyValueForUOM(uomid, uomLabel.ID, refAggrValueObj.GetPropertyValue());
								}
								else
								{
									//string uomid = refAggrValueObj.GetUOMID();
                                    string newPropValue = refAggrValueObj.GetPropertyValueForUOM(modelUOMID, uiUOMIDID, refAggrValueObj.GetPropertyValue());
                                    if (refAggrValueObj.GetDataType() == null)
                                        newPropValue = Symcon.LightRidge.PEAdvisor.GridUtilities.Utilities.FormatNumber(newPropValue, _app);
                                    smblLbl.Text = newPropValue;// refAggrValueObj.GetPropertyValueForUOM(uomid, myProp.GetUIUOMID(), refAggrValueObj.GetPropertyValue());
								}
								smblLbl.ID = refAggrValueObj.GetID();
								smblLbl.Tag	= refAggrValueObj;
								smblLbl.IsRefrence = true;
								if(propertiesMappingMode ==	GraphicsManagers.PropertiesMappingMode.NORMAL)
								{
									smblLbl.OffsetX	= (offsetXPropValue+ offsetXUOM) - offsetXPropValue/2;
									smblLbl.OffsetX =	-offsetXPropValue;
								}
                                smblLbl.BackgroundStyle.Color = refAggrValueObj.GetBackColor();
                                smblLbl.BackgroundStyle.Color = refAggrValueObj.GetForeColor();
                                
                                GraphicsUtilities.SetColor(smblLbl, refAggrValueObj);
									
								//}
								smblLbl.Name = plProp.Name + "(" + refMode.ModeName +	")";
							
							}
							else
							{
								smblLbl.Text = "";
								smblLbl.ID = "";
								smblLbl.Tag	= null;
							}

							smblLbl.ParentID = plProp.ID;
							//smblLbl.LabelDoubleClick+=new NodeMouseEventHandler(smblLbl_LabelDoubleClick);
                            
                            
							InitLabelLookFeel(smblLbl);
                            if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
                            {
                                smblLbl.OffsetY = symbolHeight;
                            }
							//smblLbl.OffsetX = wdth;

							break;
						}
						case "12":
						{
							LRRLabel smblLbl = mySymbol.Labels[_labelindex]as LRRLabel;
							//smblLbl.LabelDoubleClick-=new NodeMouseEventHandler(smblLbl_LabelDoubleClick);
							//smblLbl.Visible = true;
                            float symbolHeight = smblLbl.OffsetY;
							_labelindex++;
							if(propertiesMappingMode ==	GraphicsManagers.PropertiesMappingMode.OPEN && smblLbl.ID != "" && smblLbl.ID !=  null)
							{
								smblLbl.Tag	= uomTypeObj;//LRRModel.ChildModel.UOMs.GetUOMObjectFromID(smblLbl.ID);
								//smblLbl.LabelDoubleClick+=new NodeMouseEventHandler(smblLbl_LabelDoubleClick);
                               
                               
                                smblLbl.OffsetY = symbolHeight;
								//	smblLbl.IsUOM = true;
							}
                            else if (propertiesMappingMode != GraphicsManagers.PropertiesMappingMode.NONE)
							{
								//smblLbl.LabelDoubleClick-=new NodeMouseEventHandler(smblLbl_LabelDoubleClick);
								//smblLbl.Visible	= true;
										  //if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.NORMAL || propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.REMAP)//&& smblLbl.ParentID != plProp.SerializedID)
										  //{
										  //    smblLbl.Visible = _app.GetOptions().GraphicsLblsVisibility[(lblVsbltyIdx * 2)];
										  //}
								string myUOMName = GraphicsUtilities.GetUomName(myProp.GetUIUOMID());
								smblLbl.Text = myUOMName;
								smblLbl.Name = "UOM" + "(" + myUOMName+	")-" + myProp.GetName();
								smblLbl.ParentID = plProp.SerializedID;
								smblLbl.ID = myProp.GetUIUOMID();
								smblLbl.IsUOM = true;
								if(propertiesMappingMode ==	GraphicsManagers.PropertiesMappingMode.NORMAL)
								{
									smblLbl.OffsetX	= offsetXUOM - offsetXUOM/2;
									smblLbl.OffsetX=-offsetXUOM;
								}
								if(uomTypeObj != null)
								{
									smblLbl.Tag = uomTypeObj;//LRRModel.ChildModel.UOMs.GetUOMObjectFromID(myProp.GetUIUOMID());
									//smblLbl.LabelDoubleClick+=new NodeMouseEventHandler(smblLbl_LabelDoubleClick);
								}
								//smblLbl.UOMChanged+=new PropertyChanged(smblLbl_UOMChanged);
								InitLabelLookFeel(smblLbl);
							}
							if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.NORMAL || propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.REMAP)//&& smblLbl.ParentID != plProp.SerializedID)
							{
								smblLbl.Visible = _app.GetOptions().GraphicsLblsVisibility[(lblVsbltyIdx * 2) + 2];
							}
							break;
						}
						case "21":
						{
							LRRLabel smblLbl = mySymbol.Labels[_labelindex]as LRRLabel;
							smblLbl.Visible = true;
                            float symbolHeight = smblLbl.OffsetX;
							_labelindex++;

							if(compare2AggrValueObj != null)
							{
								if(smblLbl.Tag != null)
								{
									gm.LRRBindingManagerObj.RemoveBinding(smblLbl.Tag,"PropertyValue",smblLbl,true);
                                    gm.LRRBindingManagerObj.RemoveBinding(smblLbl.Tag, "ErrCode", smblLbl, true);
								}
									lrrBindingManager.AddBinding(compare2AggrValueObj,"PropertyValue",smblLbl,"Text");
                                    lrrBindingManager.AddBinding(compare1AggrValueObj, "ErrCode", smblLbl, "LabelColorCode");
								if(uomLabel != null)
								{
									string uomid = compare2AggrValueObj.GetUOMID();
                                    string newPropValue = compare2AggrValueObj.GetPropertyValueForUOM(uomid, uomLabel.ID, compare2AggrValueObj.GetPropertyValue());
                                    if (compare2AggrValueObj.GetDataType() == null)
                                         newPropValue = Symcon.LightRidge.PEAdvisor.GridUtilities.Utilities.FormatNumber(newPropValue, _app);
                                    smblLbl.Text = newPropValue;// compare2AggrValueObj.GetPropertyValueForUOM(uomid, uomLabel.ID, compare2AggrValueObj.GetPropertyValue());
								}
								else
								{
									//string uomid = compare2AggrValueObj.GetUOMID();
                                    string newPropValue = compare2AggrValueObj.GetPropertyValueForUOM(modelUOMID, uiUOMIDID, compare2AggrValueObj.GetPropertyValue());
                                    if (compare2AggrValueObj.GetDataType() == null)
                                        newPropValue = Symcon.LightRidge.PEAdvisor.GridUtilities.Utilities.FormatNumber(newPropValue, _app);
                                    smblLbl.Text = newPropValue;// compare2AggrValueObj.GetPropertyValueForUOM(uomid, myProp.GetUIUOMID(), compare2AggrValueObj.GetPropertyValue());
								}
								smblLbl.ID = compare2AggrValueObj.GetID();
								smblLbl.Tag = compare2AggrValueObj;
								smblLbl.IsCompare2 = true;
								if(propertiesMappingMode ==	GraphicsManagers.PropertiesMappingMode.NORMAL)
								{
									smblLbl.OffsetX	= offsetXPropValue + offsetXUOM	- offsetXPropValue/2 /* - offsetXUOM/2*/;
									smblLbl.OffsetX=-offsetXPropValue;
								}
                                smblLbl.BackgroundStyle.Color = compare2AggrValueObj.GetBackColor();
                                smblLbl.BackgroundStyle.Color = compare2AggrValueObj.GetForeColor();
                                GraphicsUtilities.SetColor(smblLbl, compare2AggrValueObj);
								smblLbl.Name = plProp.Name + "(" + compare2Mode.ModeName +	")";
								smblLbl.Visible = isCompare2Visible;
							}
							else
							{
								smblLbl.Text = "";
								smblLbl.ID = "";
								smblLbl.Tag	= null;
					
							}

							smblLbl.ParentID = plProp.SerializedID;
							//smblLbl.LabelDoubleClick+=new NodeMouseEventHandler(smblLbl_LabelDoubleClick);
							InitLabelLookFeel(smblLbl);
                            if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
                            {
                                smblLbl.OffsetY= symbolHeight;
                            }

							break;
						}

						case "22":
						{
							LRRLabel smblLbl;
							if(mySymbol.Labels.Count < 22 + (selectedProps.Count*2))
							{
								smblLbl = new LRRLabel();
								//smblLbl.Anchor = BoxPosition.TopLeft;
								mySymbol.Labels.Insert(_labelindex,smblLbl);
								//smblLbl.EditStyle.Enabled = true;
								//smblLbl.EditStyle.AllowResize = false; 
								smblLbl.HorizontalAlignment	= StringAlignment.Center;
								smblLbl.VerticalAlignment	= StringAlignment.Far;
								smblLbl.OffsetX= 10;
								smblLbl.OffsetY = 10;
								//smblLbl.LabelDoubleClick+=new NodeMouseEventHandler(smblLbl_LabelDoubleClick);
                                
                                //smblLbl.LabelDoubleClick += new NodeLabelEditEventHandler(smblLbl_LabelDoubleClick);
								//smblLbl.Translate(mySymbol.Labels[_labelindex - 1].X + mySymbol.Labels[_labelindex - 1].Width + smblLbl.OffsetX - smblLbl.OffsetX,mySymbol.Labels[_labelindex - 1].Y  - smblLbl.OffsetY);
							}
							else
							{
								smblLbl = mySymbol.Labels[_labelindex]as LRRLabel;
							}
//							smblLbl.Visible = true;
//							smblLbl.ParentID = plProp.ID;
							_labelindex++;
							//smblLbl.BackgroundStyle.Color = Color.Red;
							break;
						}
						default:
						{
							break;
						}
					}			
				}
			}
		}



		/// <summary>
		/// Given a PropertyTable of a symbol returns the selected Aggregates Properties 
		/// </summary>
		/// <param name="pt"></param>
		/// <returns>LRRAggregateProperties</returns>
		public static Symcon.BusinessObjects.CustomCollection GetSelectedAggregateProperties(PropertyTable pt,Group smbl)
		{


			Hashtable hshSelectedProps = new Hashtable();

			LRRPlantItemProperties selectedProperties = new LRRPlantItemProperties();// = new LRRSiteComponentProperties();
            try
            {
                ArrayList arryPropIndex = smbl.GetPropertyContainerByName("ArryIndexedProps") as ArrayList;
                
                for (int j = 0; j < arryPropIndex.Count; j++)
                {
                    if (arryPropIndex[j].ToString() == "Null")
                    {
                        LRRPlantItemProperty lrrscp = new LRRPlantItemProperty();
                        lrrscp.Name = "NullValue";
                        selectedProperties.Add(lrrscp);
                    }
                    else
                    {
                        ArrayList arrToRemovePropSpecs = new ArrayList();
                        Symcon.Lightridge.LRRUI.PropertyBag.PropertySpecCollection pscl = pt.Properties;
                        foreach (PropertySpec ps in pscl)
                        {
                            if (ps.ObjectRef != null && hshSelectedProps.ContainsKey(ps.ObjectRef.ToString()))
                                continue;
                            if (ps.Category == "Bound Properties")
                            {

                                LRRAggregateProperty lrrcp = LRRModel.AggregatePropertiesObj.GetAggregatePropFromID(ps.ObjectRef.ToString());
                                
                                if (lrrcp == null)
                                {
                                    return null;
                                    
                                }
                                LRRMasterProperty lrrmcp = LRRModel.MasterComponentPropertiesObj.GetMasterPropertyByID(lrrcp.MasterPropertyID);
                                if (lrrcp != null && !selectedProperties.Contains(lrrcp) && lrrmcp.IsExposed)
                                {
                                    LRRPlantItemProperty propertyObj = new LRRPlantItemProperty();
                                    propertyObj.ID = lrrcp.ID;
                                    propertyObj.PropertyID = lrrcp.ID;
                                    propertyObj.Name = lrrcp.Name;
                                  
                                    propertyObj.Description = lrrcp.Description;
                                    propertyObj.IsExposed = true;
                                    propertyObj.SerializedID = lrrcp.ID;
                                    selectedProperties.Add(propertyObj);
                                    hshSelectedProps.Add(propertyObj.ID, propertyObj);
                                    propertyObj = null;
                                }
                                else if (lrrcp != null)
                                {
                                    LRRPlantItemProperty propertyObj = new LRRPlantItemProperty();
                                    propertyObj.ID = lrrcp.ID;
                                    propertyObj.PropertyID = lrrcp.ID;
                                    propertyObj.Name = lrrcp.Name;
                                    propertyObj.Description = lrrcp.Description;
                                    propertyObj.IsExposed = false;
                                    propertyObj.SerializedID = lrrcp.ID;
                                    selectedProperties.Add(propertyObj);
                                    hshSelectedProps.Add(propertyObj.ID, propertyObj);
                                    propertyObj = null;

                                }
                                else if (ps.ObjectRef.ToString().Trim() != "")
                                {
                                    //arrToRemovePropSpecs = new ArrayList();
                                    //pscl.Remove(ps);
                                    arrToRemovePropSpecs.Add(ps);
                                    continue;
                                }
                            }
                        }
                        foreach (PropertySpec var in arrToRemovePropSpecs)
                        {

                            foreach (LRRLabel lbl in smbl.Labels)
                            {
                                if (lbl.ParentID == var.ObjectRef.ToString())
                                {
                                    lbl.Text = "";
                                    lbl.ID = "";
                                    lbl.Tag = null;
                                    continue;
                                }
                            }
                            pscl.Remove(var);
                        }
                    }
                   
                }
                return selectedProperties;
            }
			catch(Exception ex)
			{
				
				Logger.LogMessage(typeof(ImplementerScreen),"GetSelectedAggregateProperties :"+ex.Message  ,Logger.Level.Debug);
				return selectedProperties;

			}
		}



		private static PropertyTable GetPropertyTableForsymbol(Group symbol)
		{
			PropertyTable pt = symbol.GetPropertyContainerByName("PT") as	PropertyTable;
			return pt;
		}


		private static LRRAggregate GetAggregateForSymbol(Group symbol)
		{
			string siteItemID = GraphicsUtilities.GetSiteItemIdForSymbol(symbol);
			Symcon.LightRidge.BusinessObjects.LRRAggregates	lrrAggregates = LRRModel.AggregatesObj;
			LRRAggregate lrrAggregate = lrrAggregates.GetAggregateFromID(siteItemID);//Get the Aggregate By Name
			return  lrrAggregate;
		}

		/// <summary>
		/// Set The General Properties For the Label
		/// </summary>
		/// <param name="lbl"></param>
        private static void InitLabelLookFeel(Syncfusion.Windows.Forms.Diagram.Label lbl)
		{
			//			lbl.EditStyle.Enabled = true;
			//			lbl.ReadOnly = false;
			//			lbl.WrapText = true;
			//			lbl.NoClip = false;
			//			lbl.Width = 40f;
			//			lbl.Height = 15;
		}


		#endregion

		#region Fields

		private	static LRRApp	_app;
		private static LRROptions _lrrOptionsObj;
		private static int _labelindex = 1;
		private static bool flagRemove = true;
		public static event NodeMouseEventHandler LabelDoubleClick;

		#endregion

//		public static void OnUOMChanged(LRRLabel smblLabel, string propertyName)
//		{
//			Symbol smbl = smblLabel.Parent as Symbol;
//			foreach(LRRLabel lrrlabel in smbl.Labels)
//			{
//				if(lrrlabel.ParentID == smblLabel.ParentID)
//				{
//					if(lrrlabel.Tag != null && lrrlabel.Tag.GetType().GetInterface("IPropertyValue") == typeof(IPropertyValue))
//					{
//						IPropertyValue lrrscpv =	lrrlabel.Tag as	IPropertyValue;
//						string oldUomID =	lrrscpv.GetUOMID();
//						lrrlabel.Text =	GraphicsUtilities.ChangeUOM(oldUomID,propertyName,lrrlabel.Text);
//					}
//				}
//			}
//			PropertyTable myPT =  GetPropertyTableForsymbol(smbl);
//			for(int i= myPT.Properties.Count; i>0;i--)
//			{
//				PropertySpec ps =  myPT.Properties[i-1] as PropertySpec;
//				if(ps.Category == "Bound Properties")
//				{
//					if(ps.ObjectRef.ToString() == smblLabel.ParentID)
//					{
//						ps.DefaultValue = smbl.Labels[ps.Name].Text;
//					}
//				}
//			}
//			string newUOMName = LRRModel.ChildModel.GetUOMNameFromID(propertyName);
//			smblLabel.ID = propertyName;
//			smblLabel.Name.Replace(smblLabel.Text,newUOMName);
//			smblLabel.Text = newUOMName; 
//
//		}

	    private static IPropertyValue SetValueObjects(LRRMode mode, ref double currentCompareValue, LRRUOMType uomTypeObj, IProperty myProp, LRRPlantItemProperty plProp)
        {
			Symcon.LightRidge.BusinessObjects.IPropertyValue iPropVal = null;
			if (myProp.GetPropertyType() == "1")
			{
				 iPropVal = LRRModel.AggregatePropertyValuesObj.GetAggregateValueObjectForPropertyMode(mode.ModeId,myProp.GetID());
			}
		    else
			{
			    if (plProp.LookupQualifierID != null && plProp.LookupQualifierID.Trim() != "")
			    {
					  iPropVal = LRRModel.AggregateGroupedPropertyValuesObj.GetAggregateGroupedPropertyValueObjectForMode(mode.ModeId);  
			    }
			    else
			    {
					 iPropVal = LRRModel.AggregateGroupedPropertyValuesObj.GetAggregateGroupedPropertyValueObjectForMode(mode.ModeId);  
			    }
			}
			if (iPropVal != null)
			{
			    if (uomTypeObj != null && !uomTypeObj.IsEVV)
			        currentCompareValue = (double)Symcon.BusinessObjects.SymExcelConverter.Convert(typeof(string), iPropVal.GetPropertyValue(), typeof(double));//Decimal.Parse(compare1PropValueObj.PropertyValue);
			}
            return iPropVal;
        }

		/*private static void smblLbl_LabelDoubleClick(object sender, NodeMouseEventArgs evtArgs)
		{
			LRRLabel lbl = evtArgs.Node as LRRLabel;
			if(lbl != null )
			{
				try
				{
					if(lbl.ID != null && lbl.IsUOM && lbl.Tag != null)
					{
						LRRUOMType lrrUOMType = lbl.Tag as LRRUOMType;
						if(lrrUOMType != null)
							GraphicsUtilities.ChangeUOMClick(lbl,lrrUOMType.UOMTypeID);
					}
					else
					{
						if(LabelDoubleClick != null)
						{
							//LabelDoubleClick(sender,evtArgs);
                            LabelDoubleClick(evtArgs);
						}
					}
				}
				catch(Exception ex)
				{
					Logger.LogMessage(typeof(ImplementerScreen),"gms_LabelDoubleClick :"+ex.Message  ,Logger.Level.Debug);
				}
			}
		}*/

        public static void SetOverRideValue(LRRLabel lbl, string oldValue)
        {
            LRRLabel parentLabel = GraphicsUtilities.GetParentLabel(lbl);
            IProperty iProp = null;
            if (parentLabel != null)
            {
                iProp = parentLabel.Tag as IProperty;
            }
            if (iProp != null)
            {

                if (iProp.GetPropertyType() == "1")
                {
                    SetSiteCompPropOveride(lbl, oldValue);
                }
                else
                {
                    //SetGroupPropOverride(lbl, oldValue);
                }
            }
        }

        /*private static void SetGroupPropOverride(LRRLabel lbl, string oldValue)
        {
            LRRLabel parentLbl = GraphicsUtilities.GetParentLabel(lbl);
            LRRLabel uomlabel = GraphicsUtilities.GetUOMLabel(lbl);
            IProperty lrrScgp = parentLbl.Tag as IProperty;
            IPropertyValue iPropValue = lbl.Tag as IPropertyValue;
            LRRSiteGroupPropertyQualifierValue lrscgpVal = iPropValue as LRRSiteGroupPropertyQualifierValue;
            string groupPropID = lrscgpVal.GroupPropertyID;
            string qualifierID = lrscgpVal.QualifierID;
            string lookUpPropID = lrscgpVal.LookUpPropertiesID;


            LRRLookUpQualifier lkUpQualifier = LRRModel.LookUpQualifiers.GetLookUpQualifierForPropTypeAndQualifierID(qualifierID, lrscgpVal.PropertyID);
            string uomTypeID = null;
            string uomID = null;
            if (lrrScgp != null)
            {
                uomTypeID = lrrScgp.GetUOMType();
                LRRMeasurementSystemDefinitions lrrMSDs = (LRRMeasurementSystemDefinitions)_app.LRRDataStoreObj.GetDataStoreObject(typeof(LRRMeasurementSystemDefinitions));
                uomID = iPropValue.GetUOMID();// LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(LRRModel.MeasurementSystems.GetCurrentModelUOM().ID,lrrScgp.GetUOMType());
            }

            //LRRUOM uomObj = uomlabel.Tag as LRRUOM;
            string newUOM = "";

            //if(uomObj!= null)
            newUOM = uomlabel.ID;
            string newpropValue = GraphicsUtilities.ChangeUOM(newUOM, uomID, oldValue);

            if (iPropValue.SupportsOverride() && iPropValue.GetPropertyValue() != newpropValue)
            {
                bool prevOverrideState = iPropValue.GetOverrideState();
                iPropValue.SetOverrideState(newpropValue);
                bool currOverrideState = iPropValue.GetOverrideState();

                //Measurement Mode
                if (prevOverrideState != currOverrideState && iPropValue.GetModeID() == "1")
                {
                    _app.SwapGroupPropBindings(lbl.Tag, true);
                    iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
                }

                //Other Modes
                if (prevOverrideState != currOverrideState && iPropValue.GetModeID() != "1")
                {
                    _app.RemoveCascadedBindings(lbl.Tag, groupPropID, lookUpPropID, qualifierID, true);
                    iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
                }

                iPropValue.SetPropertyValue(newpropValue);

                PopulateGroupOverrides(iPropValue, groupPropID, qualifierID);
            }

        }*/

        private static void SetSiteCompPropOveride(LRRLabel lbl, string oldValue)
        {
            PathNode smbl = lbl.Container.Parent as PathNode;
            
            LRRLabel hdrLbl = smbl.Labels[0] as LRRLabel;
            LRRSiteComponent lsc = hdrLbl.Tag as LRRSiteComponent;
            LRRLabel parentLbl = GraphicsUtilities.GetParentLabel(lbl);
            IPropertyValue iPropValue = lbl.Tag as IPropertyValue;
            IProperty iProp = parentLbl.Tag as IProperty;

            string modeID = iPropValue.GetModeID();

            //string uomID = iProp.GetUOMType();
            string uomTypeID = iProp.GetUOMType();
            string uomID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(_app.AppLocaleID, uomTypeID);//iPropValue.GetUOMID();//
            LRRLabel Uomlbl = GraphicsUtilities.GetUOMLabel(lbl);
            string newUomID = "";
            string newpropValue = "";
            if (Uomlbl != null && Uomlbl.Tag != null)
            {
                //newUomID	= LRRModel.ChildModel.GetUOMIDFromName(Uomlbl.Text);

                newUomID = Uomlbl.ID;
                if(newUomID != null || newUomID != "")
                    newpropValue = GraphicsUtilities.ChangeUOM(newUomID, uomID, oldValue, iPropValue.GetDataType());
                else
                    newpropValue = GraphicsUtilities.ChangeUOM(newUomID, uomID, oldValue, iPropValue.GetDataType());
            }


            if (iPropValue.SupportsOverride() && iPropValue.GetPropertyValue() != newpropValue)
            {
                bool prevOverrideState = iPropValue.GetOverrideState();
                iPropValue.SetOverrideState(newpropValue);
                bool currOverrideState = iPropValue.GetOverrideState();

                //Measurement Mode
                if (prevOverrideState != currOverrideState && modeID == "1")
                {
                    iPropValue.SetTrustedState("-1");
                    _app.SwapBindings(lbl.Tag, true);
                    iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
                }

                LRRAggregateProperty aggProp = LRRModel.AggregatePropertiesObj.GetAggregatePropFromID(parentLbl.ID);
                string aggregateID = aggProp.AggregateID;
                   
                //Other Modes
                if (prevOverrideState != currOverrideState && modeID != "1")
                {
                    iPropValue.SetTrustedState("-1");
                    _app.RemoveCascadedBindings(lbl.Tag, typeof(LRRAggregate), parentLbl.ID, aggregateID, true);
                    iPropValue.SetOverrideValue(iPropValue.GetActualPropertyValue());
                }

                iPropValue.SetPropertyValue(newpropValue);
                if (parentLbl.Tag != null)
                {
                    PopulateOverrides(iPropValue, parentLbl.ID, aggregateID, modeID, uomID);
                }
            }
        }

        private static void PopulateOverrides(IPropertyValue iPropValue, string aggPropID, string aggregateID, string modeID, string sNewUOM)
        {
            //LRROverride lrrModelOverride = LRRModel.ChildModel.Overrides.GetOverrideObjectByPropertyAndModeID(propertyID, modeID);
            LRRAggregateOverride aggregateOverride = LRRModel.ChildModel.AggregateOverrides.GetOverrideObjectByAggPropertyAndModeID(aggPropID, modeID); 
            if (iPropValue.GetOverrideState())
            {
                if (aggregateOverride != null)
                {
                    sNewUOM = aggregateOverride.UOM;
                    LRRModel.ChildModel.AggregateOverrides.Remove(aggregateOverride);
                }
                LRRAggregateOverride lrrAggOverride = new LRRAggregateOverride();
                lrrAggOverride.ModeID = modeID;
                lrrAggOverride.AggregatePropertyID = aggPropID;
                lrrAggOverride.AggregateID = aggregateID;

                //Pre/Post Override Values
                string modelUOM = iPropValue.GetUOMID();
                lrrAggOverride.PreOverrideValue = iPropValue.GetPropertyValueForUOM(modelUOM, sNewUOM, iPropValue.GetActualOverridedValue());
                lrrAggOverride.PostOverrideValue = iPropValue.GetPropertyValueForUOM(modelUOM, sNewUOM, iPropValue.GetActualPropertyValue());

                lrrAggOverride.UOM = sNewUOM;
                //Assign Confidence Level
                string dataConfidenceLevelID = null;
                if (modeID == "1" || modeID == "2")
                {
                    dataConfidenceLevelID = "3";
                }
                else
                {
                    dataConfidenceLevelID = "1";
                }
               
                lrrAggOverride.DataConfidenceLevelID = dataConfidenceLevelID;
                LRRModel.ChildModel.AggregateOverrides.Add(lrrAggOverride);

                iPropValue.DataConfidenceValueChanged += new EventHandler(iPropValue_DataConfidenceValueChanged);


                iPropValue.SetDataConfidenceLevelID(dataConfidenceLevelID);
            }
            else
            {
                if (aggregateOverride != null)
                {
                    iPropValue.DataConfidenceValueChanged -= new EventHandler(iPropValue_DataConfidenceValueChanged);
                    LRRModel.ChildModel.AggregateOverrides.Remove(aggregateOverride);
                }
            }
            //Update Overrides
            _app.UpdateAggregateOverridePoints(); 
        }

        private static void iPropValue_DataConfidenceValueChanged(object sender, EventArgs e)
        {
            GraphicsUtilities.OnDataConfidenceValueChanged(sender, e);
        }
        public static void OnOverrideValueClickForSiteComponent(MenuItem miItem, IPropertyValue iPropValue, LRRAggregateProperty aggProperty)
        {
            if (aggProperty != null)
            {
                switch (miItem.Index)
                {
                    //Override Value On/Off
                    case 0:
                        iPropValue.SetOverrideState(false);
                        if (iPropValue.GetModeID() == "1")
                        {
                            _app.SwapBindings(iPropValue, false);
                            iPropValue.SetPropertyValue(iPropValue.GetActualOverridedValue());
                        }
                        else
                        {
                            _app.RemoveCascadedBindings(iPropValue, typeof(LRRAggregate), aggProperty.MasterPropertyID, aggProperty.AggregateID, false);
                            iPropValue.SetPropertyValue(iPropValue.GetActualOverridedValue());
                        }
                        //Populate Overrides Collection
                        IProperty iProp = aggProperty  as IProperty;
                        string propID = iProp.GetID();
                        string uomTypeID = iProp.GetUOMType();
                        string uomID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(_app.AppLocaleID, uomTypeID);//iPropValue.GetUOMID();//
                        PopulateOverrides(iPropValue, aggProperty.ID ,aggProperty.AggregateID , iPropValue.GetModeID(), uomID);//rowData.UOMID);
                        break;
                    //Uncertain Input
                    case 2:
                        iPropValue.SetTrustedState("0");
                        break;
                    //Trusted Input
                    case 3:
                        iPropValue.SetTrustedState("1");
                        break;
                    //Temp Trusted Input
                    case 4:
                        iPropValue.SetTrustedState("2");
                        break;
                    //Uncertain Override
                    case 5://3:
                        iPropValue.SetDataConfidenceLevelID("3");
                        iPropValue.SetTrustedState("-1");
                        break;
                    //Trusted Override
                    case 6://4:
                        iPropValue.SetDataConfidenceLevelID("1");
                        iPropValue.SetTrustedState("-1");
                        break;
                    //TempTrusted Override
                    case 7://5:
                        iPropValue.SetDataConfidenceLevelID("2");
                        iPropValue.SetTrustedState("-1");
                        break;
                }
            }
        }

	}
}
