using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Reflection;
using System.Diagnostics;
using System.IO;
using System.Resources;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization.Formatters;
using System.Data;
using Symcon.Utils.MessageLogger;
using Symcon.Lightridge.LRRUI.Controls;
using Symcon.Lightridge.LRRUI;
using Symcon.LightRidge.BusinessObjects;
using Symcon.LightRidge.PEAdvisor.DataStore;
using Symcon.LightRidge.Options;
using Symcon.BusinessObjects;

using Syncfusion.Windows.Forms.Diagram;




namespace Symcon.LightRidge.PEAdvisor
{
	/// <summary>
	/// General Utilities For Graphics
	/// </summary>
	public class GraphicsUtilities
	{
		static GraphicsUtilities()
		{
			_app = AppDomain.CurrentDomain.GetData("AppObject") as LRRApp;
			_lrrOptionsObj = _app.GetOptions() as LRROptions;

		}

		#region Private Methods

		/// <summary>
		/// Gets Canvas Data from the DB when opening saved View
		/// </summary>
		/// <param name="fileID"></param>
		/// <returns></returns>
		public static LRRSavedViewData GetCanvasData(string fileID)
		{
			try
			{
				string whereClause =  " WHERE ID = '"+ fileID + "'"; // Where Clause for Querying the DB
				LRRSavedViewDatas lrrSavedViewDatas = new LRRSavedViewDatas(); 
				_app.DBManagerObj.PopulateCollection(lrrSavedViewDatas,whereClause);//Fetch Saved View data using FileID
				LRRSavedViewData myobj = lrrSavedViewDatas[0];
				return myobj;
			}
			catch(Exception ex)
			{
				Logger.LogMessage(typeof(GraphicsUtilities),"GetCanvasData :"+ex.Message  ,Logger.Level.Debug);
				return null;
			}
		}

		/// <summary>
		/// Binds Each Node of the symbol to StieComponent
		/// and Shows selected Properties and Values
		/// </summary>
		/// <param name="activeCanvas"></param>
		public static void BindSymbols(GraphicsManager gm, LRRModes selectedModes,GraphicsManagers.PropertiesMappingMode propertiesMappingMode,GraphicsManagers.UserMode userMode)
		{
			Canvas activeCanvas = gm.CanvasObj;
			Group openedViewSymbol = null;
			activeCanvas.SuspendLayout();
			ArrayList arrGroupedNodes = new ArrayList();
            ArrayList toRemoveNodes = new ArrayList();
			//Find Grouped Nodes and add to arrayList -arrGroupedNodes
			foreach(INode nde in activeCanvas.Model.Nodes)
			{
				if(nde.GetType() == typeof(Syncfusion.Windows.Forms.Diagram.Group))
				{
					arrGroupedNodes.Add(nde);
				}
                if (nde.GetType().BaseType == typeof(Symbol))
                {
                    //Symbol smbl = nde as Symbol;
                    PathNode smbl = nde as PathNode;
                    
                   
                    if (smbl.PinPoint.Y == 0 && smbl.PinPoint.X== 0 && smbl.BoundingRectangle.X == 0 && smbl.BoundingRectangle.Y == 0)
                    {
                        toRemoveNodes.Add(smbl);
                    }
                }
			}

			//Ungroup Grouped Nodes 
			foreach(INode grpNode in arrGroupedNodes)
			{
				//Syncfusion.Windows.Forms.Diagram.UngroupCmd unGroup = new UngroupCmd();
                
				//unGroup.Nodes.Add(grpNode);
				//gm.CanvasObj.Controller.ExecuteCommand(unGroup);

			}
            //Delete Corrupt Nodes 
            foreach (INode wrongNode in toRemoveNodes)
            {
                gm.CanvasObj.Controller.SelectionList.Clear();
                //gm.CanvasObj.SelectionList.Add(wrongNode);
                //gm.CanvasObj.Controller.SelectionList.Add(wrongNode);
                gm.CanvasObj.Controller.Delete();
                //gm.CanvasObj.SelectionList.Clear();

            }
            toRemoveNodes.Clear();
			arrGroupedNodes.Clear();
			arrGroupedNodes = null;
			activeCanvas.ResumeLayout();


			//Iterate Through Each Symbol and add to snc
			foreach(INode nde in activeCanvas.Model.Nodes)
			{

				EditStyle editStyle =  (EditStyle)nde.GetType().GetProperty("EditStyle").GetValue(nde,null);
				if(editStyle != null && userMode == GraphicsManagers.UserMode.OPERATOR && propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
				{
					activeCanvas.FullControl = false;
					//					editStyle.AllowMove = false;
					//					editStyle.AllowResize = false;
					//					editStyle.AllowRotate = false;
					//					editStyle.AllowSelect = true;
					//					editStyle.AllowVertexEdit = false;
					editStyle.DefaultHandleEditMode = HandleEditMode.None;
					editStyle.Enabled = false;
				}

                //if(nde.GetType() == typeof(Syncfusion.Windows.Forms.Diagram.Group))
                //{
                //    arrGroupedNodes.Add(nde);
                //}

				openedViewSymbol = nde as Group;


				//string siteItemID = null;
				if(openedViewSymbol != null && openedViewSymbol.GetPropertyContainerByName("PlantItem") != null)
				{
                    //Changed By Ashwin
                    SiteItem siteItem = openedViewSymbol.GetPropertyContainerByName("PlantItem") as SiteItem;

                    if (openedViewSymbol.GetPropertyContainerByName("MasterPlantItem") != null)  // && (string)openedViewsymbol.GetPropertyContainerByName("ItemType") != "Select")
					{
						PropertyTable pt =  GetPropertyTableForsymbol(openedViewSymbol);
		
						if(openedViewSymbol.GetType().Name == "Aggregates")
						{
							CustomCollection lrrAggregateProps = GraphicsAggregatesUtilities.GetSelectedAggregateProperties(pt,openedViewSymbol);
                            if(lrrAggregateProps != null)
							GraphicsAggregatesUtilities.UpdateSymbolPropertiesMap(openedViewSymbol,selectedModes,lrrAggregateProps,gm,propertiesMappingMode,userMode,activeCanvas.CreateGraphics());
						}
                        else if (openedViewSymbol.GetType().Name == "DashBoardControlProperties")
                        {
                            CustomCollection lrrAggregateProps = GraphicsAggregatesUtilities.GetSelectedAggregateProperties(pt, openedViewSymbol);
                            if (lrrAggregateProps != null)
                            {
                                GraphicsAggregatesUtilities.UpdateSymbolPropertiesMap(openedViewSymbol, selectedModes, lrrAggregateProps, gm, propertiesMappingMode, userMode, activeCanvas.CreateGraphics());
                            }
                            else
                            {
                                CustomCollection lrrscps = GraphicsSiteComponentUtilities.GetSelectedSiteComponentProperties(pt, openedViewSymbol);
                                GraphicsSiteComponentUtilities.UpdateSymbolPropertiesMap(openedViewSymbol, selectedModes, lrrscps, gm, propertiesMappingMode, userMode, activeCanvas.CreateGraphics());
                            }
                        }
                        else
                        {
                            CustomCollection lrrscps = GraphicsSiteComponentUtilities.GetSelectedSiteComponentProperties(pt, openedViewSymbol);
                            GraphicsSiteComponentUtilities.UpdateSymbolPropertiesMap(openedViewSymbol, selectedModes, lrrscps, gm, propertiesMappingMode, userMode, activeCanvas.CreateGraphics());
                        }
					}

                   
                    // code commented and added by ashwin for custom formatting

                    string customFormatID = openedViewSymbol.GetPropertyContainerByName("CustomFormatID") as string;
                    if (customFormatID != null)
                    {
                        LRRSiteComponentPropertyValue lrrscpv = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(customFormatID);
                        if (lrrscpv != null)
                        {
                            //openedViewSymbol.BackgroundStyle.Color = lrrscpv.GetGraphicsBackColor();
                            openedViewSymbol.LineStyle.LineColor = lrrscpv.GetForeColor();
                            gm.LRRBindingManagerObj.AddBinding(lrrscpv, "PropertyValue", openedViewSymbol, "ID");
                        }
                        else
                        {
                            string scp = openedViewSymbol.GetPropertyContainerByName("SCP") as string;
                            string modeID = openedViewSymbol.GetPropertyContainerByName("ModeID") as string;
                            if (scp != null && modeID != null)
                            {
                                lrrscpv = LRRModel.PropertyValuesObj.GetPropValueBysiteCompAndModeID(scp, modeID);
                                if (lrrscpv != null)
                                {
                                    //openedViewSymbol.BackgroundStyle.Color = lrrscpv.GetGraphicsBackColor();
                                    openedViewSymbol.LineStyle.LineColor = lrrscpv.GetForeColor();
                                    gm.LRRBindingManagerObj.AddBinding(lrrscpv, "PropertyValue", openedViewSymbol, "ID");
                                }
                            }
                        }
                    }
                    
                    /*
                    string customFormatID = openedViewsymbol.GetPropertyContainerByName("CustomFormatID") as string;
                    LRRSiteComponentPropertyValue lrrscpv = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(customFormatID);
                    if (lrrscpv != null && customFormatID != null && customFormatID == lrrscpv.GetID())
                    {
                        openedViewSymbol.BackgroundStyle.Color = lrrscpv.GetGraphicsBackColor();
                        openedViewSymbol.LineStyle.LineColor = lrrscpv.GetForeColor();
                        gm.LRRBindingManagerObj.AddBinding(lrrscpv, "PropertyValue", openedViewSymbol, "ID");
                    }
                    **/
				}
				else if(openedViewSymbol != null && openedViewSymbol.GetType() == typeof(DynamicGridcontainer))
				{
                    // Cannot be cast Group using Custom Node
                    Node dg = openedViewSymbol as Node;

					//DynamicGridcontainer dg =  openedViewSymbol as DynamicGridcontainer;
                    
                    // TO get the label from Node cast to PathNode 
                    PathNode pathLabel = dg as PathNode;

                    foreach (Syncfusion.Windows.Forms.Diagram.Label lbl in pathLabel.Labels)
					{
                        
						if(lbl.GetType() == typeof(GridHeaderCell))
							continue;
						else
						{
							GridCell gCell = lbl as GridCell;

         
                            //GridCell gCell = ((PathNode)lbl).Labels[0] as GridCell;
							if(!gCell.IsBinded)
								continue;
							else
							{
								switch(gCell.Type.Name)
								{
									case "LRRSiteComponentPropertyValue":
									{
                                        LRRSiteComponentProperty prop =  LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(gCell.PropertyID);
                                        if (prop != null)
                                        {
                                            IPropertyValue iPropvalue = LRRModel.PropertyValuesObj.GetPropValueBysiteCompAndModeID(gCell.PropertyID, gCell.ModeID);
                                            gCell.Tag = iPropvalue;
                                            gm.LRRBindingManagerObj.AddBinding(iPropvalue, "PropertyValue", gCell, "Text");
                                            //Add Colour Binding
                                            gm.LRRBindingManagerObj.AddBinding(iPropvalue, "ErrCode", gCell, "LabelColorCode");
                                            if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
                                            {
                                                if (gCell.UOMTypeID != null)
                                                {
                                                    gCell.GridCellDoubleClick += new NodeMouseEventHandler(OnUOMChangeForGridCell);
                                                    gCell.GridCellDoubleClick += new NodeMouseEventHandler(gCell_GridCellDoubleClick);
                                                    //gCell.GridCellDoubleClick+=new NodeMouseEventHandler(GraphicsUtilities.gCell_GridCellDoubleClick);
                                                }
                                              
                                                gCell.IsBinded = true;
                                            }
                                            UpdateGridCell(gCell);
                                        }
                                        else
                                        {
                                            gCell.IsBinded = false;
                                            gCell.UIUOMID = null;
                                            gCell.Tag = null;
                                            gCell.UOMTypeID = null;
                                            gCell.ModeID = null;
                                            gCell.Type = null;
                                            gCell.PropertyID = null;
                                            gCell.BackgroundStyle.Color = Color.Transparent;
                                            //gCell.BorderStyle.BorderColor = dg.GridBorderColor;
                                            //gCell.BorderStyle.BorderWidth = dg.GridBorderWidth;
                                            gCell.Text = "";
                                            gCell.Description = "";
                                            gCell.FontStyle.Style = System.Drawing.FontStyle.Regular;
																		  
                                        }
										break;
									}
									case "LRRSiteGroupPropertyQualifierValue":
                                        {
                                            string lookUpQualID = gCell.ID;
                                            string groupPropID = gCell.PropertyID;
                                          //  string lookUpPropertiesID = gCell.PropertyID;
                                            //LRRModel.SiteGroupPropertyQualifierValuesObj.get
                                            LRRSiteComponentGroupProperty grpProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(groupPropID);
                                            LRRLookUpQualifier lukUPQual =  LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lookUpQualID);
                                            if (grpProp != null)
                                            {
                                                IPropertyValue iPropvalue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObject(groupPropID, lookUpQualID, gCell.ModeID);
                                                if (iPropvalue == null)
                                                {
                                                    iPropvalue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(gCell.ID);
                                                    LRRSiteGroupPropertyQualifierValue currentPropval = iPropvalue as LRRSiteGroupPropertyQualifierValue;
                                                    gCell.ID = currentPropval.LookUpQualifierID;
                                                    gCell.PropertyID = currentPropval.GroupPropertyID;
                                                }
                                                //IPropertyValue iPropvalue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(gCell.ID);
                                                gCell.Tag = iPropvalue;
                                                gm.LRRBindingManagerObj.AddBinding(iPropvalue, "PropertyValue", gCell, "Text", true);
                                                //Add Colour Binding
                                                gm.LRRBindingManagerObj.AddBinding(iPropvalue, "ErrCode", gCell, "LabelColorCode", true);
                                                if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
                                                {
                                                    if (gCell.UOMTypeID != null)
                                                    {
                                                        gCell.GridCellDoubleClick += new NodeMouseEventHandler(OnUOMChangeForGridCell);
                                                        //gCell.GridCellDoubleClick += new NodeMouseEventHandler(gCell_GridCellDoubleClick);
                                                    }
                                                    gCell.IsBinded = true;
                                                }
                                                UpdateGridCell(gCell);
                                            }
                                            else
                                            {
                                                gCell.IsBinded = false;
                                                gCell.UIUOMID = null;
                                                gCell.Tag = null;
                                                gCell.UOMTypeID = null;
                                                gCell.ModeID = null;
                                                gCell.Type = null;
                                                gCell.PropertyID = null;
                                                gCell.BackgroundStyle.Color = Color.Transparent;
                                                //gCell.BorderStyle.BorderColor = dg.GridBorderColor;
                                                //gCell.BorderStyle.BorderWidth = dg.GridBorderWidth;
                                                gCell.Text = "";
                                                gCell.Description = "";
                                                gCell.FontStyle.Style = System.Drawing.FontStyle.Regular;
                                            }
                                          
                                            break;

									}
                                case "LRRAggregatePropertyValue":
                                    {


                                        IPropertyValue iPropvalue = LRRModel.AggregatePropertyValuesObj.GetAggregateValueObjectForPropertyMode(gCell.ModeID, gCell.PropertyID);
										gCell.Tag = iPropvalue;
                                        if (iPropvalue == null)
                                        {
                                            continue;
                                        }
										gm.LRRBindingManagerObj.AddBinding(iPropvalue,"PropertyValue",gCell,"Text");
										//Add Colour Binding
										//gm.LRRBindingManagerObj.AddBinding(iPropvalue,"ErrCode",gCell,"LabelColorCode");
										if(propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
										{
											gCell.GridCellDoubleClick+=new NodeMouseEventHandler(OnUOMChangeForGridCell);
                                            //gCell.GridCellDoubleClick += new Syncfusion.Windows.Forms.Grid.GridCellClickEventHandler(OnUOMChangeForGridCell);
                                            //gCell.GridCellDoubleClick += new NodeMouseEventHandler(gCell_GridCellDoubleClick);
                                           
                                           
                                            
											gCell.IsBinded = true;
										}
										UpdateGridCell(gCell);
										break;
									}
									default:
										break;
								}

							}
						}
					}

				}
			}
		}

        static void gCell_GridCellDoubleClick(NodeMouseEventArgs evtArgs)
        {
            throw new NotImplementedException();
        }

        // bind symbols overloading added by ashwin for handling custom formatting
        public static void BindSymbols(GraphicsManager gm, LRRModes selectedModes, GraphicsManagers.PropertiesMappingMode propertiesMappingMode, GraphicsManagers.UserMode userMode,string firstTime)
        {
            Canvas activeCanvas = gm.CanvasObj;
            Group openedViewSymbol = null;
            activeCanvas.SuspendLayout();
            ArrayList arrGroupedNodes = new ArrayList();
            ArrayList toRemoveNodes = new ArrayList();
            //Find Grouped Nodes and add to arrayList -arrGroupedNodes
            foreach (INode nde in activeCanvas.Model.Nodes)
            {
                if (nde.GetType() == typeof(Syncfusion.Windows.Forms.Diagram.Group))
                {
                    arrGroupedNodes.Add(nde);
                }
                if (nde.GetType().BaseType == typeof(Symbol))
                {
                    Symbol smbl = nde as Symbol;
                    if (smbl.PinPoint.Y == 0 && smbl.PinPoint.X == 0 && smbl.PinPoint.X == 0 && smbl.PinPoint.Y == 0)
                    {
                        toRemoveNodes.Add(smbl);
                    }
                }
            }

            //Ungroup Grouped Nodes 
            foreach (INode grpNode in arrGroupedNodes)
            {
                //Syncfusion.Windows.Forms.Diagram.UngroupCmd unGroup = new UngroupCmd();
                //unGroup.Nodes.Add(grpNode);
                //gm.CanvasObj.Controller.ExecuteCommand(unGroup);

            }
            //Delete Corrupt Nodes 
            foreach (INode wrongNode in toRemoveNodes)
            {
                gm.CanvasObj.Controller.SelectionList.Clear();
                //gm.CanvasObj.SelectionList.Add(wrongNode);
                gm.CanvasObj.Controller.Delete();
                gm.CanvasObj.Controller.SelectionList.Clear();

            }
            toRemoveNodes.Clear();
            arrGroupedNodes.Clear();
            arrGroupedNodes = null;
            activeCanvas.ResumeLayout();


            //Iterate Through Each Symbol and add to snc
            foreach (INode nde in activeCanvas.Model.Nodes)
            {

                EditStyle editStyle = (EditStyle)nde.GetType().GetProperty("EditStyle").GetValue(nde, null);
                if (editStyle != null && userMode == GraphicsManagers.UserMode.OPERATOR && propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
                {
                    activeCanvas.FullControl = false;
                    //					editStyle.AllowMove = false;
                    //					editStyle.AllowResize = false;
                    //					editStyle.AllowRotate = false;
                    //					editStyle.AllowSelect = true;
                    //					editStyle.AllowVertexEdit = false;
                    editStyle.DefaultHandleEditMode = HandleEditMode.None;
                    editStyle.Enabled = false;
                }

                //if(nde.GetType() == typeof(Syncfusion.Windows.Forms.Diagram.Group))
                //{
                //    arrGroupedNodes.Add(nde);
                //}

                openedViewSymbol = nde as Group;


                //string siteItemID = null;
                if (openedViewSymbol != null && openedViewSymbol.GetPropertyContainerByName("PlantItem") != null)
                {
                    //Changed By Ashwin
                    SiteItem siteItem = openedViewSymbol.GetPropertyContainerByName("PlantItem") as SiteItem;
                    
                    if (openedViewSymbol.GetPropertyContainerByName("MasterPlantItem") != null)  // && (string)openedViewsymbol.GetPropertyContainerByName("ItemType") != "Select")
                    {
                        PropertyTable pt = GetPropertyTableForsymbol(openedViewSymbol);

                        if (openedViewSymbol.GetType().Name == "Aggregates")
                        {
                            CustomCollection lrrAggregateProps = GraphicsAggregatesUtilities.GetSelectedAggregateProperties(pt, openedViewSymbol);
                            if (lrrAggregateProps != null)
                                GraphicsAggregatesUtilities.UpdateSymbolPropertiesMap(openedViewSymbol, selectedModes, lrrAggregateProps, gm, propertiesMappingMode, userMode, activeCanvas.CreateGraphics());
                        }
                        else if (openedViewSymbol.GetType().Name == "DashBoardControlProperties")
                        {
                            CustomCollection lrrAggregateProps = GraphicsAggregatesUtilities.GetSelectedAggregateProperties(pt, openedViewSymbol);
                            if (lrrAggregateProps != null)
                            {
                                GraphicsAggregatesUtilities.UpdateSymbolPropertiesMap(openedViewSymbol, selectedModes, lrrAggregateProps, gm, propertiesMappingMode, userMode, activeCanvas.CreateGraphics());
                            }
                            else
                            {
                                CustomCollection lrrscps = GraphicsSiteComponentUtilities.GetSelectedSiteComponentProperties(pt, openedViewSymbol);
                                GraphicsSiteComponentUtilities.UpdateSymbolPropertiesMap(openedViewSymbol, selectedModes, lrrscps, gm, propertiesMappingMode, userMode, activeCanvas.CreateGraphics());
                            }
                        }
                        else
                        {
                            CustomCollection lrrscps = GraphicsSiteComponentUtilities.GetSelectedSiteComponentProperties(pt, openedViewSymbol);
                            GraphicsSiteComponentUtilities.UpdateSymbolPropertiesMap(openedViewSymbol, selectedModes, lrrscps, gm, propertiesMappingMode, userMode, activeCanvas.CreateGraphics());
                        }
                    }


                    // code commented and added by ashwin for custom formatting

                    string customFormatID = openedViewSymbol.GetPropertyContainerByName("CustomFormatID") as string;
                    if (customFormatID != null)
                    {
                        LRRSiteComponentPropertyValue lrrscpv = LRRModel.PropertyValuesObj.FindBySiteComponentPropID(customFormatID);
                        if (lrrscpv != null)
                        {
                            //openedViewSymbol.SetPropertyValue("SCP", lrrscpv.SiteComponentPropertyID);
                            //openedViewSymbol.SetPropertyValue("ModeID", "2");
                            //openedViewSymbol.BackgroundStyle.Color = lrrscpv.GetGraphicsBackColor();
                            
                            openedViewSymbol.LineStyle.LineColor = lrrscpv.GetForeColor();
                            gm.LRRBindingManagerObj.AddBinding(lrrscpv, "PropertyValue", openedViewSymbol, "ID");
                        }
                        else
                        {
                            string scp = openedViewSymbol.GetPropertyContainerByName("SCP") as string;
                            string modeID = openedViewSymbol.GetPropertyContainerByName("ModeID") as string;
                            if (scp != null && modeID != null)
                            {
                                lrrscpv = LRRModel.PropertyValuesObj.GetPropValueBysiteCompAndModeID(scp, modeID);
                                if (lrrscpv != null)
                                {
                                    //openedViewSymbol.BackgroundStyle.Color = lrrscpv.GetGraphicsBackColor();
                                    openedViewSymbol.LineStyle.LineColor = lrrscpv.GetForeColor();
                                    gm.LRRBindingManagerObj.AddBinding(lrrscpv, "PropertyValue", openedViewSymbol, "ID");
                                }
                            }
                        }
                    }
                }
                else if (openedViewSymbol != null && openedViewSymbol.GetType() == typeof(DynamicGridcontainer))
                {
                    // Cannot be cast Group using Custom Node
                    Node dg = openedViewSymbol as Node;

                    //DynamicGridcontainer dg =  openedViewSymbol as DynamicGridcontainer;

                    // TO get the label from Node cast to PathNode 
                    PathNode pathLabel = dg as PathNode;

                    foreach (Syncfusion.Windows.Forms.Diagram.Label lbl in pathLabel.Labels)
                    {
                        
                        if (lbl.GetType() == typeof(GridHeaderCell))
                            continue;
                        else
                        {
                            GridCell gCell = lbl as GridCell;
                            if (!gCell.IsBinded)
                                continue;
                            else
                            {
                                switch (gCell.Type.Name)
                                {
                                    case "LRRSiteComponentPropertyValue":
                                        {
                                            LRRSiteComponentProperty prop = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(gCell.PropertyID);
                                            if (prop != null)
                                            {
                                                IPropertyValue iPropvalue = LRRModel.PropertyValuesObj.GetPropValueBysiteCompAndModeID(gCell.PropertyID, gCell.ModeID);
                                                gCell.Tag = iPropvalue;
                                                gm.LRRBindingManagerObj.AddBinding(iPropvalue, "PropertyValue", gCell, "Text");
                                                //Add Colour Binding
                                                gm.LRRBindingManagerObj.AddBinding(iPropvalue, "ErrCode", gCell, "LabelColorCode");
                                                if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
                                                {
                                                    if (gCell.UOMTypeID != null)
                                                    {
                                                        gCell.GridCellDoubleClick += new NodeMouseEventHandler(OnUOMChangeForGridCell);
                                                        //gCell.GridCellDoubleClick += new NodeMouseEventHandler(gCell_GridCellDoubleClick);
                                                        
                                                    }

                                                    gCell.IsBinded = true;
                                                }
                                                UpdateGridCell(gCell);
                                            }
                                            else
                                            {
                                                gCell.IsBinded = false;
                                                gCell.UIUOMID = null;
                                                gCell.Tag = null;
                                                gCell.UOMTypeID = null;
                                                gCell.ModeID = null;
                                                gCell.Type = null;
                                                gCell.PropertyID = null;
                                                gCell.BackgroundStyle.Color = Color.Transparent;
                                                //gCell.BorderStyle.BorderColor = dg.GridBorderColor;
                                                //gCell.BorderStyle.BorderWidth = dg.GridBorderWidth;
                                                gCell.Text = "";
                                                gCell.Description = "";
                                                gCell.FontStyle.Style = System.Drawing.FontStyle.Regular;

                                            }
                                            break;
                                        }
                                    case "LRRSiteGroupPropertyQualifierValue":
                                        {
                                            string lookUpQualID = gCell.ID;
                                            string groupPropID = gCell.PropertyID;
                                            //  string lookUpPropertiesID = gCell.PropertyID;
                                            //LRRModel.SiteGroupPropertyQualifierValuesObj.get
                                            LRRSiteComponentGroupProperty grpProp = LRRModel.SiteComponentGroupPropertiesObj.GetSiteCompGroupPropFromID(groupPropID);
                                            LRRLookUpQualifier lukUPQual = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(lookUpQualID);
                                            if (grpProp != null)
                                            {
                                                IPropertyValue iPropvalue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetQualifierValueObject(groupPropID, lookUpQualID, gCell.ModeID);
                                                if (iPropvalue == null)
                                                {
                                                    iPropvalue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(gCell.ID);
                                                    LRRSiteGroupPropertyQualifierValue currentPropval = iPropvalue as LRRSiteGroupPropertyQualifierValue;
                                                    gCell.ID = currentPropval.LookUpQualifierID;
                                                    gCell.PropertyID = currentPropval.GroupPropertyID;
                                                }
                                                //IPropertyValue iPropvalue = LRRModel.SiteGroupPropertyQualifierValuesObj.GetSiteCompGroupPropFracValFromID(gCell.ID);
                                                gCell.Tag = iPropvalue;
                                                gm.LRRBindingManagerObj.AddBinding(iPropvalue, "PropertyValue", gCell, "Text", true);
                                                //Add Colour Binding
                                                gm.LRRBindingManagerObj.AddBinding(iPropvalue, "ErrCode", gCell, "LabelColorCode", true);
                                                if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
                                                {
                                                    if (gCell.UOMTypeID != null)
                                                    {
                                                        gCell.GridCellDoubleClick += new NodeMouseEventHandler(OnUOMChangeForGridCell);
                                                        //gCell.GridCellDoubleClick += new NodeMouseEventHandler(gCell_GridCellDoubleClick);
                                                    }
                                                    gCell.IsBinded = true;
                                                }
                                                UpdateGridCell(gCell);
                                            }
                                            else
                                            {
                                                gCell.IsBinded = false;
                                                gCell.UIUOMID = null;
                                                gCell.Tag = null;
                                                gCell.UOMTypeID = null;
                                                gCell.ModeID = null;
                                                gCell.Type = null;
                                                gCell.PropertyID = null;
                                                gCell.BackgroundStyle.Color = Color.Transparent;
                                                //gCell.BorderStyle.BorderColor = dg.GridBorderColor;
                                                //gCell.BorderStyle.BorderWidth = dg.GridBorderWidth;
                                                gCell.Text = "";
                                                gCell.Description = "";
                                                gCell.FontStyle.Style = System.Drawing.FontStyle.Regular;
                                            }

                                            break;

                                        }
                                    case "LRRAggregatePropertyValue":
                                        {


                                            IPropertyValue iPropvalue = LRRModel.AggregatePropertyValuesObj.GetAggregateValueObjectForPropertyMode(gCell.ModeID, gCell.PropertyID);
                                            gCell.Tag = iPropvalue;
                                            if (iPropvalue == null)
                                            {
                                                continue;
                                            }
                                            gm.LRRBindingManagerObj.AddBinding(iPropvalue, "PropertyValue", gCell, "Text");
                                            //Add Colour Binding
                                            //gm.LRRBindingManagerObj.AddBinding(iPropvalue,"ErrCode",gCell,"LabelColorCode");
                                            if (propertiesMappingMode == GraphicsManagers.PropertiesMappingMode.OPEN)
                                            {
                                                gCell.GridCellDoubleClick += new NodeMouseEventHandler(OnUOMChangeForGridCell);
                                                //gCell.GridCellDoubleClick += new NodeMouseEventHandler(gCell_GridCellDoubleClick);
                                                gCell.IsBinded = true;
                                            }
                                            UpdateGridCell(gCell);
                                            break;
                                        }
                                    default:
                                        break;
                                }

                            }
                        }
                    }

                }
            }

        }

		//
		//		public static void UnwirePropertyValueEvent(GraphicsManager gm)
		//		{
		//			Canvas activeCanvas = gm.CanvasObj;
		//			//LRRModes selectedModes = gms.SelectedModes;
		//
		//
		//			Symbol mySymbol = null;
		//            
		//        
		//			//Iterate Through Each Symbol and add to snc
		//			foreach(INode nde in activeCanvas.Model.Nodes)
		//			{
		//				mySymbol = nde as Symbol;
		//				string propName = null;
		//				if(mySymbol != null && mySymbol.GetPropertyContainerByName("SiteItem") != null && mySymbol.GetPropertyContainerByName("SiteItem").ToString() != "Select")
		//				{
		//
		//					if(mySymbol.GetPropertyContainerByName("SiteItem").ToString().EndsWith("]"))
		//					{
		//						propName = mySymbol.GetPropertyContainerByName("SiteItem").ToString().Substring(0,mySymbol.GetPropertyContainerByName("SiteItem").ToString().IndexOfAny("[".ToCharArray()));
		//					}
		//					else
		//					{
		//						propName = mySymbol.GetPropertyContainerByName("SiteItem").ToString();
		//					}
		//
		//					if(mySymbol.GetPropertyContainerByName("ItemType") != null    && (string)mySymbol.GetPropertyContainerByName("ItemType") != "Select")
		//					{
		//						PropertyTable pt =  GetPropertyTableForsymbol(mySymbol);
		//						Symcon.LightRidge.BusinessObjects.LRRSiteComponents lscs = LRRModel.SiteComponentsObj;
		//						LRRSiteComponent lsc = lscs.GetSiteComponentByName(propName);
		//						LRRSiteComponentProperties lrrScps =    lsc.Properties;
		//						foreach(LRRSiteComponentProperty lrrScp in lrrScps)
		//						{
		//							foreach(LRRMode mode in LRRModel.ProjectModes)
		//							{
		//								LRRSiteComponentPropertyValue lrrscpv = lrrScp.GetValueObjectForPropertyMode(mode.ModeId);
		//								//lrrscpv.PropertyValueChanged-=new PropertyChanged(gm.PropertyValueChanged);
		//							}
		//
		//
		//						}
		//						//  UpdateSymbolPropertiesMap(openedViewSymbol,selectedModes,lrrscps,gm);
		//					}
		//				}
		//			}
		//
		//
		//		}

		private static PropertyTable GetPropertyTableForsymbol(Group symbol)
		{
			PropertyTable pt = symbol.GetPropertyContainerByName("PT") as PropertyTable;
			return pt;
		}
		#endregion

		#region Public Methods

		/// <summary>
		/// Loads Canvas from The Database based on the SavedViewata ID
		/// </summary>
		/// <param name="fID"></param>
		/// <returns>Canvas</returns>
		public static void LoadSavedView(string fID,Canvas canvas)
		{
			byte[] byteArray = GetCanvasData(fID).ViewBlob;
			if(byteArray == null)
				return;
			MemoryStream ms = null;                          // Writes the BLOB to a file (*.bmp).
			try
			{
				ms = new MemoryStream(byteArray);
				if(ms != null )
				{
					canvas.LoadBinary(ms);
					ms.Close();
					byteArray = null;
				}
			}
			catch(Exception ex)
			{
				if(ms != null)
				{
					ms.Close();
				}
				Logger.LogMessage(typeof(GraphicsUtilities),"LoadSavedView :"+ex.Message  ,Logger.Level.Debug);
                
			}
		}

		/// <summary>
		/// Changes the UOM for graphics.
		/// </summary>
		/// <param name="oldUOMValue">OLD UOM Value</param>
		/// <param name="newUOMValue">NEW UOM Value</param>
		/// <param name="propVal">PropertyValue</param>
		/// <returns></returns>
		public static string ChangeUOM(string oldUOMValue,string newUOMValue,string propVal,string lrrDataType)
		{
			string newVal = "";
			if(propVal != null && propVal != "")
			{
				//UOMConversionUtility uomConv = new UOMConversionUtility();
				if(oldUOMValue == newUOMValue)
				{
					newVal = propVal;
				}
				else
				{
					if((oldUOMValue != null && oldUOMValue != "") &&
						(newUOMValue != null && newUOMValue != ""))
					{
                        newVal = Symcon.LightRidge.BusinessObjects.Utilities.UOMConversionUtility.ConvertUOM(oldUOMValue, newUOMValue, propVal, lrrDataType);

					}
				}
			}
			return newVal;
		}


		/// <summary>
		/// Deviation Rule for Labels Related to symbols
		/// </summary>
		/// <param name="myModes">LRRModes</param>
		/// <param name="mySymbol">Symbol</param>
		/// <param name="currentLabel">LRRLabel</param>
		public static void ApplyDevationRule(LRRModes myModes,PathNode mySymbol,LRRLabel currentLabel)
		{
			if(!_lrrOptionsObj.ShowLabels)
			{
				try
				{
					LRRLabel refLabel = null;//For Refrence Label
					LRRLabel compare1Label = null;//For Compare1 label
					LRRLabel compare2Label = null;//For Compare2 Label
					LRRLabel uomLabel = null;//For UOM label
					LRRUOM	currentUOMobj = null;//UOM object for the property
					LRRUOMType currentUOMTypeObj = null;//UOM type for the property
					string strUOMTypeID = null;//ID of the UOMtype


					//Check all label for a symbol and set 
					//appropriate labels for a property
					foreach(LRRLabel lbl in mySymbol.Labels)
					{
						if(lbl.ParentID == currentLabel.ParentID)
						{
							if(lbl.IsCompare1 && myModes.Count >0)
								compare1Label = lbl;
							else if(lbl.IsCompare2 && myModes.Count >2)
								compare2Label = lbl;
							else if(lbl.IsRefrence && myModes.Count >1)
								refLabel = lbl;
							else if(lbl.Tag != null && lbl.IsUOM)
								uomLabel = lbl;
						}
					}

					//Set the UOMtypeObject for the property
					if(uomLabel.Tag != null)
					{
						currentUOMTypeObj = uomLabel.Tag as LRRUOMType;// LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(strUOMTypeID);
					}


					LRRMode	compare1Mode = null;
					LRRMode	refMode = null;
					LRRMode	compare2Mode = null;
					SetModes(myModes,out compare1Mode,out refMode,out compare2Mode);
					double currentTolerence = 0;
					double currentRefValue = 0;
					double currentCompare1Value = 0;
					double currentCompare2Value = 0;
					double minValue = 0;
					double maxValue = 0 ;

			
					//					myModes.SortColumn  = "ModeTypeID";
					//					myModes.Sort();

					if(compare1Mode != null)
					{
						if(compare1Label != null && compare1Label.Text != null && compare1Label.Text.Trim() !=  "" && (currentUOMTypeObj != null && !currentUOMTypeObj.IsEVV ))
						{
							currentCompare1Value = (double)SymExcelConverter.Convert(typeof(string),compare1Label.Text,typeof(double));//Decimal.Parse(compare1Label.Text);
						}
					}
					if(refMode != null)
					{
						
						if(refLabel != null && refLabel.Text != null && refLabel.Text.Trim() !=  "" && (currentUOMTypeObj != null && !currentUOMTypeObj.IsEVV) )
						{
							currentRefValue = (double)SymExcelConverter.Convert(typeof(string),refLabel.Text,typeof(double));//Decimal.Parse(refLabel.Text);
						}
					}
					if(compare2Mode != null)
					{
						
						if(compare2Label != null && compare2Label.Text != null && compare2Label.Text.Trim() !=  "" && (currentUOMTypeObj != null && !currentUOMTypeObj.IsEVV ))
						{
							currentCompare2Value = (double)SymExcelConverter.Convert(typeof(string),compare2Label.Text,typeof(double));//Decimal.Parse(compare2Label.Text);
						}
					}

					if(currentUOMTypeObj != null && currentUOMTypeObj.Tolerance != null && !currentUOMTypeObj.IsEVV )
					{
						currentTolerence = (double)SymExcelConverter.Convert(typeof(string),currentUOMTypeObj.Tolerance,typeof(double));//Decimal.Parse(currentUOMTypeObj.Tolerance);//Decimal.Parse(currentUOMTypeObj.Tolerance);
						minValue = currentRefValue - (currentRefValue*currentTolerence);
						maxValue = currentRefValue + (currentRefValue*currentTolerence);
					}

					double tempValue = Math.Min(minValue,maxValue);
					maxValue = Math.Max(minValue,maxValue);
					minValue = tempValue;


					if(currentUOMTypeObj != null && !currentUOMTypeObj.IsEVV)
					{
						if(compare1Label != null)
						{
							if((minValue != maxValue || currentRefValue == 0)  && (currentCompare1Value >= maxValue ||  currentCompare1Value <= minValue))
							{
								compare1Label.Visible = true;
							}
							else
							{
								compare1Label.Visible = false;
							}
						}

						if(compare2Label != null)
						{
							if((minValue != maxValue || currentRefValue == 0) && (currentCompare2Value >= maxValue ||  currentCompare2Value <= minValue))
							{
								compare2Label.Visible = true;
							}
							else
							{
								compare2Label.Visible = false;
							}
						}
					}
				}
				catch(Exception ex)
				{
					Logger.LogMessage(typeof(GraphicsUtilities),"ApplyDevationRule :"+ex.Message  ,Logger.Level.Debug);

				}
			}
		}


		public static void SetModes(LRRModes myModes,out LRRMode compare1Mode,out LRRMode refMode,out LRRMode compare2Mode)
		{
			compare1Mode = null;
			refMode = null;
			compare2Mode = null;

			foreach(LRRMode mode in myModes)
			{
				if(mode != null)
				{
					//Set Compare1ValueObject
					if(mode.ModeTypeID == "1")
					{
						compare1Mode =	mode;
						//						compare1PropValueObj = myProp.GetValueObjectForPropertyMode(compare1Mode.ModeId);
						//						if(compare1PropValueObj	!= null	&& compare1PropValueObj.PropertyValue != null && compare1PropValueObj.PropertyValue.Trim() != ""  && !uomTypeObj.IsEVV )
						//						{
						//							currentCompare1Value = Decimal.Parse(compare1PropValueObj.PropertyValue);
						//						}
					}
					else if(mode.ModeTypeID == "2")
					{
						//Set RefValueObject
						if(mode != null)
						{
							refMode	=  mode;
							//							refPropValueObj	= myProp.GetValueObjectForPropertyMode(refMode.ModeId);
							//							if(refPropValueObj != null && refPropValueObj.PropertyValue	!= null	&& refPropValueObj.PropertyValue.Trim()	!= "" && !uomTypeObj.IsEVV)
							//							{
							//								currentRefValue	= Decimal.Parse(refPropValueObj.PropertyValue);
							//							}
						}
					}
					else if(mode.ModeTypeID == "3")
					{
						//Set Compare2ValueObject
						if(mode != null)
						{
							compare2Mode =	mode;
							//							compare2PropValueObj = myProp.GetValueObjectForPropertyMode(compare2Mode.ModeId);
							//							if(compare2PropValueObj	!= null	&& compare2PropValueObj.PropertyValue != null &&  compare2PropValueObj.PropertyValue.Trim()	!= "" && !uomTypeObj.IsEVV)
							//							{
							//								currentCompare2Value = Decimal.Parse(compare2PropValueObj.PropertyValue);
							//							}
						}
					}
				}
			}

		}

	


		/// <summary>
		/// DataConfidence level for a property is changed
		/// </summary>
		/// <param name="sender">object</param>
		/// <param name="e">EventArgs</param>
		public static void OnDataConfidenceValueChanged(object sender, EventArgs e)
		{
			IPropertyValue iPropValue = sender as IPropertyValue;
			string modeID = iPropValue.GetModeID();
			string propertyID = iPropValue.GetPropertyID();
			LRROverride lrrModelOverride = LRRModel.ChildModel.Overrides.GetOverrideObjectByPropertyAndModeID(propertyID,modeID);
			if(lrrModelOverride != null)
				lrrModelOverride.DataConfidenceLevelID = iPropValue.GetDataConfidenceLevelID();

			//Update Overrides
			_app.UpdateOverridePoints();
		}


		/// <summary>
		/// Sets the color and style for the Label based on error code
		/// </summary>
		/// <param name="lbl">LRRLabel</param>
		/// <param name="lrrscpv">LRRSiteComponentPropertyValue</param>
        public static void SetColor(Syncfusion.Windows.Forms.Diagram.Label lbl, IPropertyValue lrrscpv)
		{
			//to Change
			//lbl.BackgroundStyle.Color = lrrscpv.GetGraphicsBackColor();
            bool evvColorFormatApplied = false;

            #region for handling evvs
            string scpID = lrrscpv.GetPropertyID();
            LRRSiteComponentProperty scp = LRRModel.SiteComponentPropertiesObj.GetSiteCompPropFromID(scpID);
            if (scp != null)
            {
                string uomTypeID = scp.UOMTypeID;
                if (uomTypeID != null && uomTypeID.Trim() != "")
                {
                    LRRUOMType lrrUOMType = LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(uomTypeID);

                    if (lrrUOMType != null && lrrUOMType.IsEVV)
                    {
                        ArrayList evvs = LRRModel.ChildModel.EvvDictionary.Get(lrrUOMType.UOMTypeID);
                        if (evvs != null)
                        {
                            for (int i = 0; i < evvs.Count; i++)
                            {
                                EVVsObject evvObj = evvs[i] as EVVsObject;
                                if (evvObj.EVVValue == lrrscpv.GetPropertyValue())
                                {
                                    LRREVV evv = LRRModel.ChildModel.EVVs.GetEVVObjectFromID(evvObj.EVVID);
                                    int colInt = evv.Color;
                                    if (colInt != 0)
                                    {
                                        System.Drawing.Color color = System.Drawing.Color.FromArgb(colInt);
                                        //lrrSiteCompPropValue.BackColor = color;
                                        lbl.BackgroundStyle.Color = color;
                                        evvColorFormatApplied = true;
                                    }
                                }
                            }
                        }

                    }
                }
            }
            #endregion

            if(!evvColorFormatApplied)
			    lbl.BackgroundStyle.Color = lrrscpv.GetGraphicsBackColor();

			lbl.BackgroundStyle.Color = lrrscpv.GetForeColor();
            
			//Set Border Color and Border Thickness
			Color borderColor = lrrscpv.GetBorderColor();
			int borderThickness = lrrscpv.GetBorderThickness();
            if (borderThickness == 2)
            {
                 
                //lbl.BorderStyle.ShowBorder = true;
                //lbl.BorderStyle.BorderWidth = 2;
                //lbl.BorderStyle.BorderColor = borderColor;
            }
            else
            {
                //lbl.BorderStyle.ShowBorder = true;
                //lbl.BorderStyle.BorderWidth = borderThickness;
                //lbl.BorderStyle.BorderColor = borderColor;

            }
			//			else
			//			{
			//				lbl.BorderStyle.ShowBorder = false;
			//			}
			lbl.FontStyle.Underline = lrrscpv.GetUnderLine();
        }


		#endregion

		#region Fields

		private static LRRApp   _app;
		private  static LRROptions				_lrrOptionsObj;

		#endregion

		public static void ChangeUOMClick(LRRLabel lbl,string uomTypeId)
		{
			string selectedUOMID = lbl.ID;
            Group smbl = lbl.Container.Parent as Group;
			string siteItemName = GraphicsUtilities.GetSiteItemNameForSymbol(smbl);
			siteItemName = siteItemName + ":  " + lbl.Name.Substring(lbl.Name.LastIndexOf("-")+1);

			UOMChangeDialog uomChangeDialog = new UOMChangeDialog(uomTypeId,selectedUOMID,siteItemName);
					
			if( uomChangeDialog.ShowDialog(_app.MainFrame) ==  DialogResult.OK)
			{
				OnUOMChanged(lbl,uomChangeDialog.SelectedUOMID);
			}
		}

		public static LRRLabel GetParentLabel(LRRLabel lbl)
		{
			if(lbl != null)
			{
				PathNode smbl=lbl.Container.Parent as PathNode;
				foreach(LRRLabel prntLbl in	smbl.Labels	)
				{
					if(prntLbl.ParentID == lbl.ParentID && prntLbl.IsParent)
					{
						return prntLbl;
					}
				}
			}
			return null;

		}

		public static LRRLabel GetUOMLabel(LRRLabel	lbl)
		{
			if(lbl != null)
			{
                PathNode smbl = lbl.Container.Parent as PathNode;
				foreach(LRRLabel uomLbl	in	smbl.Labels	)
				{
					if(uomLbl.ParentID != null && uomLbl.ParentID.Trim() != "" && uomLbl.ParentID == lbl.ParentID && uomLbl.IsUOM)
					{
						return uomLbl;
					}
				
				}
			}
			return null;
		}

		public static LRRLabel GetUOMLabelFromParentID(Group symbol, string parentID)
		{
           
			foreach(LRRLabel lbl in symbol.Labels)
			{
				if(lbl.ParentID == parentID)
				{
					LRRLabel uomLabel = GetUOMLabel(lbl);
					return uomLabel;
				}
			}
			return null;
		}

		///	<summary>
		///	Get	The	UOM	name for the Site Component	Property
		///	</summary>
		///	<param name="siteCompProp"></param>
		///	<returns>string</returns>
		public static string GetUomName(string uomID)
		{
			string uomName ="";
			if(uomID !=	null &&	uomID != "")
				uomName	= LRRModel.ChildModel.GetUOMNameFromID(uomID);
			return uomName;
		}


		public static string GetSiteItemIdForSymbol(Group sm)
		{
			string siteItemID = null;
			SiteItem si = sm.GetPropertyContainerByName("PlantItem") as SiteItem;
			siteItemID = si.ID;
			return siteItemID;
		}


		public static string GetSiteItemNameForSymbol(Group sm)
		{
			string siteItemName = null;
            SiteItem si = sm.GetPropertyContainerByName("PlantItem") as SiteItem;
			if(si != null)
				siteItemName = si.Name;
			if(siteItemName == null)
				siteItemName = "";
			return siteItemName;
		}


		public	static void	OnUOMChanged(LRRLabel smblLabel, string propertyName)
		{
			try
			{
                Group smbl = smblLabel.Container.Parent as Group;
				LRRLabel parentLabel =  GraphicsUtilities.GetParentLabel(smblLabel);
				IProperty iProp = parentLabel.Tag as IProperty;
				
				foreach(LRRLabel lrrlabel in smbl.Labels)
				{
					if(lrrlabel.ParentID ==	smblLabel.ParentID)
					{
						if(lrrlabel.Tag	!= null	&& lrrlabel.Tag.GetType().GetInterface("IPropertyValue") == typeof(IPropertyValue))
						{
							IPropertyValue lrrscpv =	lrrlabel.Tag as	IPropertyValue;
							string oldUOMID = lrrscpv.GetUOMID();//LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(LRRModel.MeasurementSystems.GetCurrentModelUOM().ID,iProp.GetUOMType());
                            string newPropertyValue = GraphicsUtilities.ChangeUOM(oldUOMID, propertyName, lrrscpv.GetPropertyValue(), lrrscpv.GetDataType());
                            string lrrDataType = lrrscpv.GetDataType();
                            
                            if (lrrDataType == null || lrrDataType.Trim() == "")
                            {
                                newPropertyValue = Symcon.LightRidge.PEAdvisor.GridUtilities.Utilities.FormatNumber(newPropertyValue, _app);
                            }
                            lrrlabel.Text = newPropertyValue;// GraphicsUtilities.ChangeUOM(oldUOMID, propertyName, lrrscpv.GetPropertyValue());
							_app.LRRUIManagerObj.CheckTrendUOMChanged(iProp,lrrscpv.GetModeID(),propertyName);
                            if (smbl.GetType() == typeof(Symcon.Lightridge.LRRUI.Controls.DashBoardControlProperties))
                            {
                                ControlNode cNode = smbl.Nodes.Last as ControlNode;
                               
                                Control objControl = cNode.HostingControl;
                                objControl.GetType().GetProperty("TargetValue").SetValue(objControl, newPropertyValue, null);
                            }
						}
					}
				}
				PropertyTable myPT =  GetPropertyTableForsymbol(smbl);
				for(int	i= myPT.Properties.Count; i>0;i--)
				{
					PropertySpec ps	=  myPT.Properties[i-1]	as PropertySpec;
					if(ps.Category == "Bound Properties")
					{
						if(ps.ObjectRef.ToString() == smblLabel.ParentID)
						{
							ps.DefaultValue	= smbl.Labels[ps.Name].Text;
						}
					}
				}
				string newUOMName =	LRRModel.ChildModel.GetUOMNameFromID(propertyName);
				smblLabel.ID = propertyName;
				smblLabel.Name.Replace(smblLabel.Text,newUOMName);
				smblLabel.Text = newUOMName; 
				
			}
			catch(Exception	ex)
			{
				Logger.LogMessage(typeof(GraphicsSiteComponentUtilities),"smblLbl_UOMChanged :"+ex.Message  ,Logger.Level.Debug);
			}
		}
       
        public static void OnUOMChangeForGridCell(NodeMouseEventArgs evtArgs)
		{
			GridCell selectedCell = sender as GridCell;
            
            //GridCell selectedCell = evtArgs.Node as GridCell;
			IPropertyValue lrrscpv = selectedCell.Tag as IPropertyValue;
			if(selectedCell != null)
			{
                if (selectedCell.UOMTypeID != null)
                {


                    UOMChangeDialog uomChangeDialog = new UOMChangeDialog(selectedCell.UOMTypeID, selectedCell.UIUOMID, selectedCell.Description);
                    string oldUOMID = lrrscpv.GetUOMID();
                    PropertyTable myPT = selectedCell.PT;

                    if (DialogResult.OK == uomChangeDialog.ShowDialog(_app.MainFrame))
                    {
                        string newUOMID = uomChangeDialog.SelectedUOMID;
                        string newPropertyValue = GraphicsUtilities.ChangeUOM(oldUOMID, newUOMID, lrrscpv.GetPropertyValue(), lrrscpv.GetDataType());
                        string lrrDataType = lrrscpv.GetDataType();
                        if (lrrDataType == null || lrrDataType.Trim() == "")
                        {
                            newPropertyValue = Symcon.LightRidge.PEAdvisor.GridUtilities.Utilities.FormatNumber(newPropertyValue, _app);
                        }
                        selectedCell.Text = newPropertyValue;//GraphicsUtilities.ChangeUOM(oldUOMID,newUOMID,lrrscpv.GetPropertyValue());
                        for (int i = myPT.Properties.Count; i > 0; i--)
                        {
                            PropertySpec ps = myPT.Properties[i - 1] as PropertySpec;
                            if (ps.Category == "Bound Properties")
                            {
                                ps.DefaultValue = selectedCell.Text;
                            }
                        }
                        string newUOMName = LRRModel.ChildModel.GetUOMNameFromID(newUOMID);
                        selectedCell.UIUOMID = newUOMID;
                    }
                }
			}
		}

		public static void UpdateGridCell(GridCell gCell)
		{
			string newUOMID = gCell.UIUOMID;
			IPropertyValue lrrscpv = gCell.Tag as IPropertyValue;
            gCell.Text = GraphicsUtilities.ChangeUOM(lrrscpv.GetUOMID(), newUOMID, lrrscpv.GetPropertyValue(), lrrscpv.GetDataType());
			PropertyTable myPT = gCell.PT;
			for(int	i= myPT.Properties.Count; i>0;i--)
			{
				PropertySpec ps	=  myPT.Properties[i-1]	as PropertySpec;
				if(ps.Category == "Bound Properties")
				{
					ps.DefaultValue	= gCell.Text;
					Attribute[]	attrib = new Attribute[1];
					BrowsableAttribute browsableAttrib;
					browsableAttrib = new BrowsableAttribute(false);
					attrib[0] =	browsableAttrib;
					ps.AttributesNew = attrib;
                    break;
				}
			}
			string newUOMName =	LRRModel.ChildModel.GetUOMNameFromID(newUOMID);
			gCell.UIUOMID= newUOMID;

		}

		
		public static void OnVeiwTrend(string trendScheme, LRRLabel selectedLbl)
		{
			IProperty iProp = null;
			string modeID = null;
			string uomID = null;

			if(selectedLbl != null && selectedLbl.Tag != null)
			{
				IPropertyValue iPropVal = selectedLbl.Tag as IPropertyValue;
				if(iPropVal != null)
					modeID = iPropVal.GetModeID();
				LRRLabel parentLabel = GraphicsUtilities.GetParentLabel(selectedLbl);
				LRRLabel uomLabel = GraphicsUtilities.GetUOMLabel(selectedLbl);
				if(parentLabel != null && parentLabel.Tag != null) 
				{
					iProp = parentLabel.Tag as IProperty;
				}
				if(uomLabel != null) 
				{
					uomID = uomLabel.ID;
				}
			}

			if(iProp != null && modeID != null && uomID != null)
			{
				_app.LRRUIManagerObj.ShowTrend(trendScheme,iProp,modeID,uomID);
			}
		}

		public static void GetUIUOMIDForProperty(IProperty myProp,LRRPlantItemProperty plProp,ref string uiUOMIDID, ref string  _lrrUOMType)
		{
			LRRUOMType uomTypeObj = null;
			
			if(myProp.GetPropertyType() == "1" || myProp.GetPropertyType() == null)
			{
				//For Simple Properties
				if(myProp.GetUOMType()	!= null)
				{
					uomTypeObj = LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(myProp.GetUOMType());
				}
			}
			else
			{
				//For Grouped Properties
				if(plProp.LookupQualifierID != null && plProp.LookupQualifierID.Trim() != "")
				{
					LRRLookUpQualifier lrrLookUpQualifier = LRRModel.LookUpQualifiers.getLookUpQualifierForLkupQualifierID(plProp.LookupQualifierID);
					if(lrrLookUpQualifier != null)
						uomTypeObj = LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(lrrLookUpQualifier.UOMTypeID);


				}
					//For Data Arrays
				else
				{
					LRRPropertyType lrrPropType = LRRModel.ChildModel.PropertyTypes.getPropertyTypeForID(myProp.GetPropertyType());
					if(lrrPropType!= null)
						uomTypeObj = LRRModel.ChildModel.UomTypes.GetUOMTypeObjectFromID(lrrPropType.UOMTypeID);
				}
			}

			//Set Model and UI UOM id's
			if(uomTypeObj != null)
			{

				string modelID = LRRModel.MeasurementSystems.GetCurrentModelUOM().ID;
				string localID = _app.AppLocaleID;
				_lrrUOMType = uomTypeObj.UOMTypeID;
				//modelUOMID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(modelID,uomTypeObj.UOMTypeID);
				uiUOMIDID = LRRModel.ChildModel.MeasSysDef.GetLocaleUOM(localID,uomTypeObj.UOMTypeID);
			}
		}


		public static void ActivateToolTip(NodeMouseEventArgs evtArgs,ToolTip _toolTip,GraphicsManagers gms)
		{
			if(_app.GetOptions().ToolTipOptions == Globals.Enums.ToolTipOptions.OFF)
			{
				_toolTip.RemoveAll();
				return;
			}

			if(evtArgs.Node.GetType() == typeof(LRRLabel))
			{
				if(_app.GetOptions().ToolTipOptions == Globals.Enums.ToolTipOptions.DETAIL)
				{
					GraphicsUtilities.SetDetailToolTipForLabel(evtArgs,_toolTip,gms);
				
				}
				else if (_app.GetOptions().ToolTipOptions == Globals.Enums.ToolTipOptions.SUMMARY)
				{
					GraphicsUtilities.SetSummaryToolTipForLabel(evtArgs,_toolTip,gms);
				}
			}
			else if (evtArgs.Node.GetType().BaseType == typeof(Symbol))
			{
				if(_app.GetOptions().ToolTipOptions == Globals.Enums.ToolTipOptions.DETAIL)
				{
					GraphicsUtilities.SetDetailToolTipForSymbol(evtArgs,_toolTip,gms);
				}
				else if (_app.GetOptions().ToolTipOptions == Globals.Enums.ToolTipOptions.SUMMARY)
				{
					GraphicsUtilities.SetSummaryToolTipForSymbol(evtArgs,_toolTip,gms);

				}
			}
			else if(evtArgs.Node.GetType() == typeof(GridCell))
			{
				if(_app.GetOptions().ToolTipOptions == Globals.Enums.ToolTipOptions.DETAIL)
				{
					GraphicsUtilities.SetDetailToolTipForGridCell(evtArgs,_toolTip,gms);
				}
				else if (_app.GetOptions().ToolTipOptions == Globals.Enums.ToolTipOptions.SUMMARY)
				{
					GraphicsUtilities.SetSummaryToolTipForGridCell(evtArgs,_toolTip,gms);
				}
			}
		}

		/// <summary>
		/// Sets and Displays Detail Tool tip fo a Symbol
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="evtArgs"></param>
		/// <param name="_toolTip"></param>
		/// <param name="gms"></param>
		private static void SetDetailToolTipForSymbol(NodeMouseEventArgs evtArgs,ToolTip _toolTip,GraphicsManagers gms)
		{
			_toolTip.Active = false;
            EquipmentProperties sm = evtArgs.Node as EquipmentProperties;
            Group gp = evtArgs.Node as Group;
            //SiteItem si;
            //GridCell gc;
            //Node sm = evtArgs.Node as Node;
			Type typeOfSymbol = sm.GetType();
			string toolTipText  = "Not Bound";
			if(typeOfSymbol != typeof(Symcon.Lightridge.LRRUI.Controls.Aggregates) && typeOfSymbol != typeof(DynamicGridcontainer)) 
			{
                if (sm.GetPropertyContainerByName("MasterPlantItem") != null)
				{
					MasterPlantItem mp =sm.GetPropertyContainerByName("MasterPlantItem") as MasterPlantItem;
					if(mp != null)
					{
						toolTipText = mp.Name;
						if(sm.GetPropertyContainerByName("PlantItem") != null)
						{
                            string temp = ((Symcon.Lightridge.LRRUI.Controls.EquipmentProperties)(sm)).PlantItem.ID;
                            //string temp = ((Symcon.Lightridge.LRRUI.Controls.EquipmentProperties)).PlantItem.ID;
                            //string temp = gc.PlantItem.ID;
                            LRRSiteComponent sc = LRRModel.SiteComponentsObj.GetSiteComponentByID(temp);
                            string siteItemName1 = GraphicsUtilities.GetSiteItemNameForSymbol(gp);
                            string siteItemName = sc.Name; 
							//toolTipText = toolTipText + ": \n" ;
							toolTipText = toolTipText + ": " +  siteItemName;
							toolTipText = toolTipText + "\n" ;
						}
						PropertyTable pt =sm.GetPropertyContainerByName("PT") as PropertyTable;
						
						string propName = "";
						foreach(PropertySpec ps in pt.Properties)
						{
							if(ps.Category == "Bound Properties")
							{
								string[] modeNameArr = ps.Name.Split('(');
								string modeName = modeNameArr[1].TrimEnd(')');
								if(modeNameArr[0] != propName)
								{

									toolTipText = toolTipText + "\n";
									toolTipText = toolTipText.Substring(0,toolTipText.Length - 1);
									toolTipText = toolTipText + "\n";
									propName = modeNameArr[0];
									toolTipText = toolTipText + propName;
									if(ps.ObjectRef != null)
									{
                                        LRRLabel uomLabel = GraphicsUtilities.GetUOMLabelFromParentID(gp, ps.ObjectRef.ToString());
										if(uomLabel != null)
										{
											toolTipText = toolTipText + "[" + uomLabel.Text + "]\n";
										}
									}
								}

								//string propName = ps.Name.Substring(ps.Name.IndexOf('('));
								//propName = propName[1].Substring(0,propName.Length -1);

								toolTipText = toolTipText +  modeName  + ":" + ps.DefaultValue + "\n" ;
							}
						}
					}
				}
			}
			else
			{
                if (sm.GetPropertyContainerByName("MasterPlantItem") != null)
                {
                    MasterPlantItem mp =sm.GetPropertyContainerByName("MasterPlantItem") as MasterPlantItem;
                    if (mp != null)
                    {
                        toolTipText = mp.Name;
                        toolTipText = toolTipText + ": \n";

                        string currentAggragateName = GraphicsUtilities.GetSiteItemNameForSymbol(gp);
                        if (currentAggragateName != "")
                        {
                            toolTipText = toolTipText + currentAggragateName;
                            toolTipText = toolTipText + ": \n";
                            LRRAggregate currentAggragate = LRRModel.AggregatesObj.GetAggregateFromName(currentAggragateName);
                            currentAggragate.AggregateEquipments.SortColumn = "EquipmentID";
                            currentAggragate.AggregateEquipments.Sort();

                            foreach (LRRAggregateEquipment currentAggragateEqupnemt in currentAggragate.AggregateEquipments)
                            {
                                LRRSiteComponent currentSiteComponent = LRRModel.SiteComponentsObj.GetSiteComponentByID(currentAggragateEqupnemt.EquipmentID);
                                toolTipText = toolTipText + currentSiteComponent.Name;
                                toolTipText = toolTipText + " \n";
                            }
                            PropertyTable pt =sm.GetPropertyContainerByName("PT") as PropertyTable;
                            string propName = "";
                            foreach (PropertySpec ps in pt.Properties)
                            {
                                if (ps.Category == "Bound Properties")
                                {
                                    string[] modeNameArr = ps.Name.Split('(');
                                    string modeName = modeNameArr[1].TrimEnd(')');
                                    if (modeNameArr[0] != propName)
                                    {

                                        toolTipText = toolTipText + "\n";
                                        toolTipText = toolTipText.Substring(0, toolTipText.Length - 1);
                                        toolTipText = toolTipText + "\n";
                                        propName = modeNameArr[0];
                                        toolTipText = toolTipText + propName;
                                        if (ps.ObjectRef != null)
                                        {
                                            LRRLabel uomLabel = GraphicsUtilities.GetUOMLabelFromParentID(gp, ps.ObjectRef.ToString());
                                            if (uomLabel != null)
                                            {
                                                toolTipText = toolTipText + "[" + uomLabel.Text + "]\n";
                                            }
                                        }
                                    }
                                    toolTipText = toolTipText + modeName + ":" + ps.DefaultValue + "\n";
                                }
                            }
                        }
                    }
                }

            }
                    _toolTip.SetToolTip(gms.ActiveManager.CanvasObj,toolTipText);
			        _toolTip.Active = true;

		}


		/// <summary>
		/// Sets and Displays Summary Tool tip fo a Symbol
		/// </summary>
		/// <param name="evtArgs"></param>
		/// <param name="_toolTip"></param>
		/// <param name="gms"></param>
		private static void SetSummaryToolTipForSymbol(NodeMouseEventArgs evtArgs,ToolTip _toolTip,GraphicsManagers gms)
		{
			_toolTip.Active = false;
			Group sm = evtArgs.Node as Group;
			Type typeOfSymbol = sm.GetType();
			System.Text.StringBuilder sb = new System.Text.StringBuilder();
			sb.Append("Not Bound");

			if(typeOfSymbol != typeof(DynamicGridcontainer)) 
			{
				if(sm.GetPropertyContainerByName("MasterPlantItem") != null)
				{
					MasterPlantItem mp =sm.GetPropertyContainerByName("MasterPlantItem") as MasterPlantItem;
					if(mp != null)
					{
						sb.Remove(0,sb.Length);
						sb.Append(mp.Name);
						if(sm.GetPropertyContainerByName("PlantItem") != null)
						{
							string siteItemName = GraphicsUtilities.GetSiteItemNameForSymbol(sm);
							sb.Append(": ");
							sb.Append(siteItemName);//Split('[')[0] + " ";
						}
						/*
						PropertyTable pt =sm.GetPropertyContainerByName("PT") as PropertyTable;
						string propName = "";
						foreach(PropertySpec ps in pt.Properties)
						{
							if(ps.Category == "Bound Properties")
							{
								string[] modeNameArr = ps.Name.Split('(');
								string modeName = modeNameArr[1].TrimEnd(')');
								if(modeNameArr[0] != propName)
								{

									toolTipText = toolTipText.Substring(0,toolTipText.Length - 1);
									toolTipText = toolTipText + "\n";
									propName = modeNameArr[0];
									toolTipText = toolTipText + propName;
									if(ps.ObjectRef != null)
									{
										LRRLabel uomLabel = GraphicsUtilities.GetUOMLabelFromParentID(sm,ps.ObjectRef.ToString());
										if(uomLabel != null)
										{
											toolTipText = toolTipText + "[" + uomLabel.Text + "]\n";
										}
									}
								}
								LRRMode lrrmode = LRRModel.ProjectModes.GetModeByName(modeName );
								string modeText = "";

								if(lrrmode.ModeTypeID == "1")
								{
									modeText = "C1";
								}
								else if(lrrmode.ModeTypeID == "2")
								{
									modeText = "Ref";
								}
								else if(lrrmode.ModeTypeID == "3")
								{
									modeText = "C2";
								}
								toolTipText = toolTipText + modeText + ":" + ps.DefaultValue + "\\" ;
							}
						}
						*/
					}
				}
			}
			_toolTip.SetToolTip(gms.ActiveManager.CanvasObj,sb.ToString());
			_toolTip.Active = true;
		}


		/// <summary>
		/// Sets and Displays Detail Tool tip fo a GridCell
		/// </summary>
		/// <param name="evtArgs"></param>
		/// <param name="_toolTip"></param>
		/// <param name="gms"></param>
		private static void SetDetailToolTipForGridCell(NodeMouseEventArgs evtArgs,ToolTip _toolTip,GraphicsManagers gms)
		{
			//GridCell gridCell = evtArgs.Node as GridCell;
            GridCell gridCell=((PathNode)evtArgs.Node).Labels[0] as GridCell;
            
			System.Text.StringBuilder sb = new System.Text.StringBuilder();
			sb.Append(gridCell.Text);
			_toolTip.RemoveAll();
			MasterPlantItem mp = gridCell.MasterPlantItem;
            string temp = gridCell.PlantItem.ID;

			if(mp != null && gridCell.IsBinded)
			{
				sb.Remove(0,sb.Length);
				sb.Append(mp.Name);
                //if (gridCell.GetPropertyContainerByName(("PlantItem") != null))
                  if(gridCell.GetPropertyContainerByName("PlantItem")!=null)
				{
					string siteItemName = null;
                    SiteItem si = gridCell.GetPropertyContainerByName("PlantItem") as SiteItem;

                    LRRSiteComponent sc = LRRModel.SiteComponentsObj.GetSiteComponentByID(temp);
                    if (sc != null)
                        siteItemName = sc.Name;
                    // commented by ashwin
					//if(si != null)
						//siteItemName = si.Name;
					if(siteItemName == null)
						siteItemName = "";
					sb.Append(": " );
					sb.Append(siteItemName);
					sb.Append("\n");
					if(si.SiteItemType == Globals.Enums.PlantItemType.AGGREGATE)
					{
						LRRAggregate currentAggragate = LRRModel.AggregatesObj.GetAggregateFromName(si.Name);
						currentAggragate.AggregateEquipments.SortColumn = "EquipmentID";
						currentAggragate.AggregateEquipments.Sort();
						sb.Append("AGGREGATE OF:");
						sb.Append("\n");

						foreach(LRRAggregateEquipment currentAggragateEqupnemt in  currentAggragate.AggregateEquipments)
						{
							LRRSiteComponent currentSiteComponent =	LRRModel.SiteComponentsObj.GetSiteComponentByID(currentAggragateEqupnemt.EquipmentID);
							sb.Append(currentSiteComponent.Name);
							sb.Append("\n");
						}
					}
				}
				sb.Append("\n");

				sb.Append(gridCell.Description + " : " + gridCell.Text);

                if (gridCell.ModeID == "1")
                {
                    LRRHistPropCompPropMapping histmapping = LRRModel.HistPropMaps.GetHistMapObjFromValID(gridCell.PropertyID);
                    if (histmapping != null)
                    {
                        sb.Append("\n");
                        sb.Append("Historian Tag : " + histmapping.HistTag);
                    }
                }
				PropertyTable pt = gridCell.PT;
                
				IPropertyValue iPropVal = gridCell.Tag as IPropertyValue;
				string modelUOMID =  iPropVal.GetUOMID();
				LRRUOM modelUOM = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(gridCell.UIUOMID);
				if(modelUOM != null)
					sb.Append(" " + modelUOM.Name);
			}
			_toolTip.SetToolTip(gms.ActiveManager.CanvasObj,sb.ToString());
			_toolTip.Active = true;
		}


		/// <summary>
		/// Sets and Displays Summary Tool tip fo a GridCell
		/// </summary>
		/// <param name="evtArgs"></param>
		/// <param name="_toolTip"></param>
		/// <param name="gms"></param>
		private static void SetSummaryToolTipForGridCell(NodeMouseEventArgs evtArgs,ToolTip _toolTip,GraphicsManagers gms)
		{
			//GridCell gridCell = evtArgs.Node as GridCell;
            GridCell gridCell = ((PathNode)evtArgs.Node).Labels[0] as GridCell;
			System.Text.StringBuilder sb = new System.Text.StringBuilder();
			sb.Append(gridCell.Text);
			_toolTip.RemoveAll();
			MasterPlantItem mp = gridCell.MasterPlantItem;
			if(mp != null && gridCell.IsBinded)
			{
				sb.Remove(0,sb.Length);
				sb.Append(mp.Name);
                if (gridCell.GetPropertyContainerByName("PlantItem") != null)
				{
					string siteItemName = null;
                    SiteItem si = gridCell.GetPropertyContainerByName("PlantItem") as SiteItem;
					if(si != null)
						siteItemName = si.Name;
					if(siteItemName == null)
						siteItemName = "";
					sb.Append(siteItemName);
					sb.Append("\n");
				}
				sb.Append(gridCell.Description + " : " + gridCell.Text);
				PropertyTable pt = gridCell.PT;
				IPropertyValue iPropVal = gridCell.Tag as IPropertyValue;
				string modelUOMID =  iPropVal.GetUOMID();
				LRRUOM modelUOM = LRRModel.ChildModel.UOMs.GetUOMObjectFromID(gridCell.UIUOMID);
				if(modelUOM != null)
					sb.Append(" " + modelUOM.Name);
			}
			_toolTip.SetToolTip(gms.ActiveManager.CanvasObj,sb.ToString());
			_toolTip.Active = true;
		}

		
		/// <summary>
		/// Sets and Displays Summary Tool tip fo a Label
		/// </summary>
		/// <param name="evtArgs"></param>
		/// <param name="_toolTip"></param>
		/// <param name="gms"></param>
		private static void SetSummaryToolTipForLabel(NodeMouseEventArgs evtArgs,ToolTip _toolTip,GraphicsManagers gms)
		{
			_toolTip.Active = false;
			System.Text.StringBuilder sb = new System.Text.StringBuilder();
            LRRLabel currentLbl = ((PathNode)evtArgs.Node).Labels[0] as LRRLabel;
			LRRLabel parentLabel = GetParentLabel(currentLbl);
			LRRLabel uomLabel = GetUOMLabel(currentLbl);
			PathNode currentSymbol = parentLabel.Container.Parent as PathNode;
			IProperty iProp =  parentLabel.Tag as IProperty;
			string propName = iProp.GetName();
			sb.Append(propName);
			sb.Append("[" + uomLabel.Text + "]\n");

			foreach(LRRLabel lbl in currentSymbol.Labels)
			{
				if(lbl.ParentID == parentLabel.ParentID)
				{
					if(lbl.IsCompare1)
					{
						sb.Append("C1:" + lbl.Text + "\\ ");
					}
					if(lbl.IsRefrence)
					{
						sb.Append("Ref:" + lbl.Text +"\\ ");
					}
					if(lbl.IsCompare2)
					{
						sb.Append("C2:" + lbl.Text);
					}
				}
			}

			_toolTip.SetToolTip(gms.ActiveManager.CanvasObj,sb.ToString());
			_toolTip.Active = true;
		}


		/// <summary>
		/// Sets and Displays Detail Tool tip fo a Label
		/// </summary>
		/// <param name="evtArgs"></param>
		/// <param name="_toolTip"></param>
		/// <param name="gms"></param>
		private static void SetDetailToolTipForLabel(NodeMouseEventArgs evtArgs,ToolTip _toolTip,GraphicsManagers gms)
		{
			_toolTip.Active = false;
            LRRLabel currentLbl = ((PathNode)evtArgs.Node).Labels[0] as LRRLabel;
			System.Text.StringBuilder sb = new System.Text.StringBuilder();
			LRRLabel parentLabel = GetParentLabel(currentLbl);
			LRRLabel uomLabel = GetUOMLabel(currentLbl);
            if (parentLabel == null)
            {
                return;
            }
            PathNode currentSymbol = parentLabel.Container.Parent as PathNode;
			IProperty iProp =  parentLabel.Tag as IProperty;
            if (iProp != null)
            {
                string propName = iProp.GetName();
                sb.Append(propName);
                sb.Append("[" + uomLabel.Text + "]\n");
            }

			foreach(LRRLabel lbl in currentSymbol.Labels)
			{
				if(lbl.ParentID == parentLabel.ParentID)
				{
					if(lbl.IsCompare1)
					{
						string modeName = lbl.Name.Substring(lbl.Name.IndexOf('('));
						modeName = modeName.Replace("(","");
						modeName = modeName.Replace(")","");
						sb.Append(modeName + ": " + lbl.Text + "\n");
					}
					if(lbl.IsRefrence)
					{
						string modeName = lbl.Name.Substring(lbl.Name.IndexOf('('));
						modeName = modeName.Replace("(","");
						modeName = modeName.Replace(")","");
						sb.Append(modeName + ": " + lbl.Text + "\n");
					}
					if(lbl.IsCompare2)
					{
						string modeName = lbl.Name.Substring(lbl.Name.IndexOf('('));
						modeName = modeName.Replace("(","");
						modeName = modeName.Replace(")","");
						sb.Append(modeName + ": " + lbl.Text);
					}
				}
			}
			_toolTip.SetToolTip(gms.ActiveManager.CanvasObj,sb.ToString());
			_toolTip.Active = true;
		}


        public static void ResetLabels(Group smbl)
        {
            for (int i = smbl.Labels.Count - 1; i >= 0; i--)
            {
                smbl.Labels.Remove(smbl.Labels[i]);
            }
            smbl.Labels.Clear();
            Symcon.Lightridge.LRRUI.Controls.SymbolsUtils.CreateLabels(smbl);
			   ArrayList alst = smbl.GetPropertyContainerByName("ArryIndexedProps") as ArrayList;
				for(int i = 0;i<alst.Count;i++)
				{
				alst[i] = "Null";
				}
        }

        /// <summary>
        /// Removes the properties from property table.
        /// </summary>
        /// <param name="mySymbol">My symbol.</param>
        public static void RemoveBoundProperties(Group mySymbol)
        {
            PropertyTable pt = (PropertyTable)mySymbol.GetPropertyContainerByName("PT");
            RemoveBoundProperties(pt);
            return;
        }

        /// <summary>
        /// Removes the bound properties.
        /// </summary>
        /// <param name="pt">The pt.</param>
        public static void RemoveBoundProperties(PropertyTable pt)
        {
            for (int i = pt.Properties.Count - 1; i > 0; i--)
            {
                PropertySpec ps = pt.Properties[i] as PropertySpec;
                if (ps.Category == "Bound Properties")
                {
                    pt.Properties.Remove(ps);
                }
            }
        }

        //public static void ResetLabels(Symbol mysymbol)
        //{
        //    ResetHeader(mysymbol.Labels[0] as LRRLabel);

        //    int count = 0;
        //    int _maxRow = 3;
        //    int _maxCol = 3;
        //    for (int m = 0; m < 4; m++)
        //    {
        //        for (int i = 0; i < _maxRow; i++)
        //        {
        //            for (int j = 0; j < _maxCol; j++)
        //            {
        //                string str = (string)i.ToString(Symcon.BusinessObjects.SymConverter.Culture) + j.ToString(Symcon.BusinessObjects.SymConverter.Culture);
        //                //-------------------------------------------------
        //                //|	00(Nothing)		01(PropValue 1)		02(Nothing)
        //                //|	10(PropName)	11(PropValue 2)		12(UOM)
        //                //|	20(Property	Description)		21(PropValue 3)		22(Nothing)
        //                //-------------------------------------------------

        //                switch (str)
        //                {
        //                    case "00":
        //                        {
        //                            break;
        //                        }
        //                    case "01":
        //                        {
        //                            LRRLabel smblLbl = new LRRLabel();
        //                            smblLbl.Anchor = BoxPosition.TopLeft;
        //                            smblLbl.OffsetX = -40;
        //                            smblLbl.OffsetY = (count + (count * 45));
        //                            smblLbl.HorizontalAlignment = StringAlignment.Far;
        //                            mysymbol.Labels.Add(smblLbl);
        //                            InitLabelLookFeel(smblLbl);
        //                            break;
        //                        }

        //                    case "02":
        //                        {
        //                            LRRLabel smblLbl = new LRRLabel();
        //                            smblLbl.Anchor = BoxPosition.TopLeft;
        //                            smblLbl.OffsetX = -10;
        //                            smblLbl.OffsetY = (count + (count * 45));
        //                            smblLbl.HorizontalAlignment = StringAlignment.Far;
        //                            mysymbol.Labels.Add(smblLbl);

        //                            InitLabelLookFeel(smblLbl);
        //                            smblLbl.OffsetX = 10;
        //                            smblLbl.OffsetY = 10;
        //                            smblLbl.EditStyle.Enabled = true;
        //                            smblLbl.EditStyle.AllowResize = false;
        //                            break;
        //                        }
        //                    case "10":
        //                        {
        //                            LRRLabel smblLbl = new LRRLabel();
        //                            smblLbl.Anchor = BoxPosition.TopLeft;
        //                            smblLbl.OffsetX = -80;
        //                            smblLbl.OffsetY = (count + (count * 45)) + 15;
        //                            smblLbl.HorizontalAlignment = StringAlignment.Far;
        //                            mysymbol.Labels.Add(smblLbl);
        //                            InitLabelLookFeel(smblLbl);
        //                            break;
        //                        }
        //                    case "11":
        //                        {
        //                            LRRLabel smblLbl = new LRRLabel();
        //                            smblLbl.Anchor = BoxPosition.TopLeft;
        //                            smblLbl.OffsetX = -40;
        //                            smblLbl.OffsetY = (count + (count * 45)) + 15;
        //                            smblLbl.HorizontalAlignment = StringAlignment.Far;
        //                            mysymbol.Labels.Add(smblLbl);
        //                            InitLabelLookFeel(smblLbl);
        //                            break;
        //                        }
        //                    case "12":
        //                        {
        //                            LRRLabel smblLbl = new LRRLabel();
        //                            smblLbl.Anchor = BoxPosition.TopLeft;
        //                            smblLbl.OffsetX = -10;
        //                            smblLbl.OffsetY = (count + (count * 45)) + 15;
        //                            smblLbl.HorizontalAlignment = StringAlignment.Near;
        //                            mysymbol.Labels.Add(smblLbl);
        //                            InitLabelLookFeel(smblLbl);
        //                            smblLbl.OffsetX = 20;
        //                            break;
        //                        }
        //                    case "21":
        //                        {
        //                            LRRLabel smblLbl = new LRRLabel();
        //                            smblLbl.Anchor = BoxPosition.TopLeft;
        //                            smblLbl.OffsetX = -40;
        //                            smblLbl.OffsetY = (count + (count * 45)) + 30;
        //                            smblLbl.HorizontalAlignment = StringAlignment.Far;
        //                            mysymbol.Labels.Add(smblLbl);
        //                            InitLabelLookFeel(smblLbl);
        //                            break;
        //                        }
        //                    case "22":
        //                        {
        //                            LRRLabel smblLbl = new LRRLabel();
        //                            smblLbl.Anchor = BoxPosition.TopLeft;
        //                            smblLbl.OffsetX = -10;
        //                            smblLbl.OffsetY = (count + (count * 45)) + 30;
        //                            smblLbl.HorizontalAlignment = StringAlignment.Far;
        //                            mysymbol.Labels.Add(smblLbl);

        //                            InitLabelLookFeel(smblLbl);
        //                            smblLbl.OffsetX = 10;
        //                            smblLbl.OffsetY = 10;
        //                            smblLbl.EditStyle.Enabled = true;
        //                            smblLbl.EditStyle.AllowResize = false;
        //                            break;
        //                        }
        //                    default:
        //                        {
        //                            break;
        //                        }
        //                }
        //            }
        //        }
        //        count++;
        //    }
        //    //CreateLinkLabel(mysymbol);

        //}

        //private static void ResetHeader(LRRLabel hsmblLbl)
        //{
        //    hsmblLbl.Anchor = BoxPosition.TopLeft;
        //    hsmblLbl.OffsetY = -15;
        //    hsmblLbl.HorizontalAlignment = StringAlignment.Center;
        //    hsmblLbl.VerticalAlignment = StringAlignment.Center;
        //    InitLabelLookFeel(hsmblLbl);
        //  //  hsmblLbl.BackgroundStyle.Color = mySymbol.BackgroundStyle.Color;
        //    hsmblLbl.OffsetX = -60;
        //    hsmblLbl.OffsetX = 100;
        //}
        //private static void InitLabelLookFeel(SymbolLabel lbl)
        //{
        //    lbl.BackgroundStyle.Color = Color.Transparent;
        //    lbl.FontStyle.Name = "Arial";
        //    lbl.FontStyle.Family = "Arial";
        //    lbl.FontStyle.PointSize = 9f;
        //    lbl.EditStyle.Enabled = true;
        //    lbl.WrapText = false;
        //    lbl.NoClip = false;
        //    lbl.Width = 40f;
        //    lbl.Height = 15;
        //}
	}
}